﻿// .NET
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Media;
using System.Xml.Serialization;

namespace ADSTester
{
    public class AdsRegister : INotifyPropertyChanged
    {
        private enum EnumChange { None, NewExt, NewValue, NewBits }
        private EnumChange change = EnumChange.None;

        public AdsRegister()
        {
            bits = new OneBit[8];
            for(int i = 0; i < 8; i++) { bits[i] = new OneBit(); }
        }

        // nazev
        [XmlAttribute("Name")]
        public string Name { get; set; }
        // adresa
        [XmlAttribute("Address")]
        public byte Address { get; set; }

        // externi nastaveni / cteni
        [XmlAttribute("Value")]
        public byte ValueExt
        {
            set
            {
                SetBits(value);             // nastaveni nove hodnoty z 'byte'
                change = EnumChange.NewExt; // zmena je od 'Ext'
                Aktualizuj();               // aktualizace zobrazeni
            }
            get { return GetBits(); }       // vraceni hodnoty jako 'byte'
        }

        // hodnota z / do GUI
        [XmlIgnore]
        public string ValueGUI
        {
            get { change = EnumChange.None; return GetBits().ToString("X2"); }
            set
            {
                // puvodni hodnota
                byte valueTemp = GetBits();
                // pokud s vyzvednuti hodnoty problem
                try { SetBits(Byte.Parse(value, System.Globalization.NumberStyles.AllowHexSpecifier)); }
                // tak se pouzije puvodni hodnota
                catch { SetBits(valueTemp); }
                Aktualizuj();
                // pokud toto nebyla zmena od 'Ext' nebo 'Bits'
                if ((change != EnumChange.NewExt) && (change != EnumChange.NewBits))
                    // tak je zmena od 'Val' a zavola se aktualizace zobrazeni
                    { change = EnumChange.NewValue; Aktualizuj(); }
            }
        }
        // bits
        [XmlIgnore] public bool D7 { get { return GetBitValue(7); } set { SetBitValue(7, value); Aktualizuj(); } }
        [XmlIgnore] public bool D6 { get { return GetBitValue(6); } set { SetBitValue(6, value); Aktualizuj(); } }
        [XmlIgnore] public bool D5 { get { return GetBitValue(5); } set { SetBitValue(5, value); Aktualizuj(); } }
        [XmlIgnore] public bool D4 { get { return GetBitValue(4); } set { SetBitValue(4, value); Aktualizuj(); } }
        [XmlIgnore] public bool D3 { get { return GetBitValue(3); } set { SetBitValue(3, value); Aktualizuj(); } }
        [XmlIgnore] public bool D2 { get { return GetBitValue(2); } set { SetBitValue(2, value); Aktualizuj(); } }
        [XmlIgnore] public bool D1 { get { return GetBitValue(1); } set { SetBitValue(1, value); Aktualizuj(); } }
        [XmlIgnore] public bool D0 { get { return GetBitValue(0); } set { SetBitValue(0, value); Aktualizuj(); } }

        // texty pro bits
        public string T7 { get { return GetBitText(7); } }
        public string T6 { get { return GetBitText(6); } }
        public string T5 { get { return GetBitText(5); } }
        public string T4 { get { return GetBitText(4); } }
        public string T3 { get { return GetBitText(3); } }
        public string T2 { get { return GetBitText(2); } }
        public string T1 { get { return GetBitText(1); } }
        public string T0 { get { return GetBitText(0); } }
        // nastaveni textu
        [XmlAttribute("Texts")]
        public string Texts
        {
            set
            {
                string[] texts = value.Split(' '); Array.Reverse(texts);
                for(int i = 7;  i >= 0; i--) { try { bits[i].Text = texts[i]; } catch { bits[i].Text = "---"; } }
            }
            get { return String.Empty; }
        }

        // poznamka
        [XmlAttribute("Note")] public string Note { get; set; }

        // --- ---

        // aktualizace (radeji) vsech polozek s hodnotou
        private void Aktualizuj()
        {
            onPropertyChanged(this, "ValueGUI");
            onPropertyChanged(this, "D7");
            onPropertyChanged(this, "D6");
            onPropertyChanged(this, "D5");
            onPropertyChanged(this, "D4");
            onPropertyChanged(this, "D3");
            onPropertyChanged(this, "D2");
            onPropertyChanged(this, "D1");
            onPropertyChanged(this, "D0");
        }

        // --- ---

        // Declare the PropertyChanged event
        public event PropertyChangedEventHandler PropertyChanged;

        // OnPropertyChanged will raise the PropertyChanged event passing the
        // source property that is being updated.
        private void onPropertyChanged(object sender, string propertyName)
        {
            if (this.PropertyChanged != null) { PropertyChanged(sender, new PropertyChangedEventArgs(propertyName)); }
        }

        public bool Enb7 { get { return GetBitEnable(7); } }
        public bool Enb6 { get { return GetBitEnable(6); } }
        public bool Enb5 { get { return GetBitEnable(5); } }
        public bool Enb4 { get { return GetBitEnable(4); } }
        public bool Enb3 { get { return GetBitEnable(3); } }
        public bool Enb2 { get { return GetBitEnable(2); } }
        public bool Enb1 { get { return GetBitEnable(1); } }
        public bool Enb0 { get { return GetBitEnable(0); } }

        // maska/ prednastaveni
        [XmlAttribute("Mask")]
        public string Mask
        {
            get { return String.Empty; }
            set
            {
                char[] chars = value.ToCharArray();
                Array.Reverse(chars);
                for(int i = 7; i >= 0; i--)
                {
                    if (chars[i] != '.')
                    {
                        // pokud neni '.' tak je bit nejak blokovan / prednastaven
                        bits[i].state = OneBit.EnumState.Disable;
                        if (chars[i] == '0') { bits[i].PreSet = false; }
                        if (chars[i] == '1') { bits[i].PreSet = true; }
                        if (chars[i] == 'R') { /* ReadOnly */ }
                    }
                }
            }
        }

        // --- ---

        private OneBit[] bits = null;

        private string GetBitText(int bitNumber) { return bits[bitNumber].Text; }
        private bool GetBitEnable(int bitNumber) { return ((bits[bitNumber].state & OneBit.EnumState.Disable) == OneBit.EnumState.None); }

        private bool GetBitValue(int bitNumber) { return bits[bitNumber].Value; }
        private void SetBitValue(int bitNumber, bool value) { bits[bitNumber].Value = value; }

        // 'byte' -> 'Bit(s)'
        private void SetBits(byte value)
        {
            for(int i = 7; i >= 0; i--) { bits[i].Value = ((value & (byte)(1 << i)) != 0x00); }
        }
        // 'Bit(s)' -> 'byte'
        private byte GetBits()
        {
            byte value = 0;
            for (int i = 7; i >= 0; i--) { if (bits[i].Value == true) { value |= (byte)(1 << i); } }
            return value;
        }

        // --- barvou pro kanal se obarvi policko adresy registru  ---

        [XmlIgnore]
        public Color Barva { get; set; }
        [XmlIgnore]
        public Brush BackChannel { get { return XUtils.Graphics.SolidColorBrushes.GetByColor(Barva); } }
    }

    public class Registers
    {
        [XmlArray("Items")]
        [XmlArrayItem("Register", typeof(AdsRegister))]
        public List<AdsRegister> regs = new List<AdsRegister>();
    }

    public class OneBit
    {
        [Flags]
        public enum EnumState { None = 0, Disable = 1 /* pouze pro cteni 8*/, PreSet = 2 /* prednastaven */ }

        public string Text = "---";
        public EnumState state = EnumState.None;
        private bool bit = false;

        public bool PreSet { set { bit = value; state |= EnumState.PreSet; } }

        public bool Value
        {
            get { return bit; }
            set { if ((state & EnumState.PreSet) == EnumState.None) { bit = value; } }
        }
    }

    // --- ---

    public class StoreRegValues
    {
        [XmlArray("Values")]
        [XmlArrayItem("Value", typeof(StoreRegValue))]
        public List<StoreRegValue> regs = new List<StoreRegValue>();
    }

    public class StoreRegValue
    {
        public byte Address { set; get; }
        public byte Value { set; get; }
    }
}
