﻿//#define ADS_VER_A
#define ADS_VER_B

// .NET
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Win32;
// XUtils
using XUtils.Graph.VersionV1;
// Commun
using Commun.AppExt;
using Commun.Global;
using Commun.Protocols.TextTypeA;

// 000 -> 16KHz
// 001 -> 8KHz
// 010 -> 4KHz
// 011 -> 2KHz
// 100 -> 1KHz
// 101 -> 500Hz
// 110 -> 250Hz
// 111 -> nepouzito
// HR=1 -> 2x rychlost

namespace ADSTester
{
    internal class GlobalConsts
    {
        public const string AppNameStr = "ADS Tester";
        public const string AppVersionStr = " [V01.01.000]";

        // pridavne povely do HW
        public enum EnumDeviceExtCmdsExt : byte { MeasureParms = 51 }

        // mezni hodnoty pro rozsah 16bits / 24bits
        public const int ConstValue16bitsMin = -0x00007FFF; public const int ConstValue16bitsMax = +0x00007FFF; // 16bits
        public const int ConstValue24bitsMin = -0x007FFFFF; public const int ConstValue24bitsMax = +0x007FFFFF; // 24bits
    }

    internal class AdsConfigV02
    {
        private const int ChannelsTotal = 8;    // maximalni pocet kanalu
        private static int actChannels = 0;     // prave aktivni kanaly bitove priznaky
        private static bool bits16or24 = false; // 16bits (false) / 24bits (true)

        private static List<string> infos = new List<string>();
        private static int infosPos = 0;

        private static PaketToSend paketToSend = new PaketToSend() { IdentMax = 99 };
        private static PaketToReceive paketToReceive = null;
        private static ObservableCollection<AdsRegister> registry = new ObservableCollection<AdsRegister>();

        public static Window MainWindow { get; set; }

        public static void Init(DataGrid dg, string regsName)
        {
            string directory = System.IO.Path.Combine(XUtils.DirsAndFiles.SpecialDirs.GetDirectory(XUtils.DirsAndFiles.SpecialDirs.EnumLocationType.LocationExe),
                "Setup", "Registers");
            Registers registers = XUtils.Xml.XmlSerialization.XmlFileToClass<Registers>(directory, regsName);
            if (registers != null)
            {
                registers.regs.All(item => { registry.Add(item); return true; });
                dg.ItemsSource = registry;
            }
            MainWindow.Title = GlobalConsts.AppNameStr + GlobalConsts.AppVersionStr;

            UtilsSimpleGraph.opticalZoomStep = 10;
            UtilsSimpleGraph.opticalZoomStepShift = 100;
            UtilsSimpleGraph.opticalShiftStep = 1000;
            UtilsSimpleGraph.opticalShiftStepShift = 5000;

            // --- prijem ---
            // prijem pomoci tri vlaken
            #if ADS_VER_A
            //  Commun.Connects.USBDevice.CntUSBDevice01.RecvBufferLowSizeUsed = 1024;
            //  Commun.Connects.USBDevice.CntUSBDevice01.RecvBufferLowSizeUsed = 1024 * 10;
            //  CommunAppExt.Commun.iConnect.ReceiveType = ConstsGlobal.EnumReceiveType.ByThreeThreats;
            #endif
            #if ADS_VER_B
              Commun.Setup.StaticSetupApp.SetupCntPrmsItems.ConnectEnable = ConstsCommun.ConnectType.COMVCP;
              Commun.Setup.StaticSetupApp.SetupCntPrmsItems.ConnectEnable = ConstsCommun.ConnectType.COMVCP;
              //CommunAppExt.Commun.iConnect.ReceiveType = ConstsGlobal.EnumReceiveType.ByOneThreat;
            #endif

            //AppCommunExtMain.ReceiveTimeOut = 2000;
            CommunAppExt.OnReceivePaketNameX += OnReceivePaketNameX;

            // polozky pro zobrazeni v grafu
            UtilsSimpleGraph.CreateGraphItems(
                new UtilsSimpleGraphItem() { Name = "Channel1", PenColor = UtilsSimpleGraph.GetColor(0), ValueMin = GlobalConsts.ConstValue16bitsMin, ValueMax = GlobalConsts.ConstValue16bitsMax },
                new UtilsSimpleGraphItem() { Name = "Channel2", PenColor = UtilsSimpleGraph.GetColor(1), ValueMin = GlobalConsts.ConstValue16bitsMin, ValueMax = GlobalConsts.ConstValue16bitsMin },
                new UtilsSimpleGraphItem() { Name = "Channel3", PenColor = UtilsSimpleGraph.GetColor(2), ValueMin = GlobalConsts.ConstValue16bitsMin, ValueMax = GlobalConsts.ConstValue16bitsMin },
                new UtilsSimpleGraphItem() { Name = "Channel4", PenColor = UtilsSimpleGraph.GetColor(3), ValueMin = GlobalConsts.ConstValue16bitsMin, ValueMax = GlobalConsts.ConstValue16bitsMin },
                new UtilsSimpleGraphItem() { Name = "Channel5", PenColor = UtilsSimpleGraph.GetColor(4), ValueMin = GlobalConsts.ConstValue16bitsMin, ValueMax = GlobalConsts.ConstValue16bitsMin },
                new UtilsSimpleGraphItem() { Name = "Channel6", PenColor = UtilsSimpleGraph.GetColor(5), ValueMin = GlobalConsts.ConstValue16bitsMin, ValueMax = GlobalConsts.ConstValue16bitsMin },
                new UtilsSimpleGraphItem() { Name = "Channel7", PenColor = UtilsSimpleGraph.GetColor(6), ValueMin = GlobalConsts.ConstValue16bitsMin, ValueMax = GlobalConsts.ConstValue16bitsMin },
                new UtilsSimpleGraphItem() { Name = "Channel8", PenColor = UtilsSimpleGraph.GetColor(7), ValueMin = GlobalConsts.ConstValue16bitsMin, ValueMax = GlobalConsts.ConstValue16bitsMin });
        }

        // --- ---

        // otevreni / pripojeni zarizeni
        public static bool DevOpen(TextBlock tbDeviceInfo)
        {
            // vymazni puvodnich zazamu 'Info'
            infos.Clear();
            //return true;

            if (CommunAppExt.Start() == false)
            {
                MessageBox.Show("Zarizeni nenalezeno!", "!!! Chyba !!!");
                tbDeviceInfo.Text = "---"; return false;
            }

            //return true;

            StringBuilder sb = new StringBuilder(); string strTemp = null;

            // vycteni informaci 'InfoDev'
            paketToSend.BeginName(ConstsPaket.Name.Cmd);
            paketToSend.AddDataNameAndValues("V", (byte)ConstsPaket.EnumCmd.Info);
            paketToSend.AddDataNameAndValues("X", (byte)ConstsPaket.EnumInfoSubCmd.Dev);
            paketToSend.End();
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck /* | CommunExtAppConsts.EnumRecv.NoWaitToAnswer */) == ConstsCommun.EnumSendEndRecvStatus.OK)
            {
                strTemp = paketToReceive.GetDataStringByName("P"); // vyrobce
                if (strTemp != null) { sb.Append("Prod=").Append(strTemp).Append(" / "); }
                strTemp = paketToReceive.GetDataStringByName("N"); // nazev
                if (strTemp != null) { sb.Append("Name=").Append(strTemp); }
                // ulozeni do seznamu 'Info(s)'
                infos.Add("Dev: " + sb.ToString());
                sb.Clear();
            }
            // vycteni informaci 'InfoHW'
            paketToSend.BeginName(ConstsPaket.Name.Cmd);
            paketToSend.AddDataNameAndValues("V", (byte)ConstsPaket.EnumCmd.Info);
            paketToSend.AddDataNameAndValues("X", (byte)ConstsPaket.EnumInfoSubCmd.HW);
            paketToSend.End();
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck /* | CommunExtAppConsts.EnumRecv.NoWaitToAnswer */) == ConstsCommun.EnumSendEndRecvStatus.OK)
            {
                strTemp = paketToReceive.GetDataStringByName("T"); // typ
                if (strTemp != null) { sb.Append("Type=").Append(strTemp).Append(" / "); }
                strTemp = paketToReceive.GetDataStringByName("V"); // verze
                if (strTemp != null) { sb.Append("Ver=").Append(strTemp).Append(" / "); }
                strTemp = paketToReceive.GetDataStringByName("R"); // revize
                if (strTemp != null) { sb.Append("Rev=").Append(strTemp).Append(" / "); }
                strTemp = paketToReceive.GetDataStringByName("U"); // Uid
                if (strTemp != null) { sb.Append("Uid=").Append(strTemp); }
                // ulozeni do seznamu 'Info(s)'
                infos.Add("HW: " + sb.ToString());
                sb.Clear();
            }
            // vycteni informaci 'InfoSW'
            paketToSend.BeginName(ConstsPaket.Name.Cmd);
            paketToSend.AddDataNameAndValues("V", (byte)ConstsPaket.EnumCmd.Info);
            paketToSend.AddDataNameAndValues("X", (byte)ConstsPaket.EnumInfoSubCmd.SW);
            paketToSend.End();
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck /* | CommunExtAppConsts.EnumRecv.NoWaitToAnswer */) == ConstsCommun.EnumSendEndRecvStatus.OK)
            {
                strTemp = paketToReceive.GetDataStringByName("T"); // typ
                if (strTemp != null) { sb.Append("Type=").Append(strTemp).Append(" / "); }
                strTemp = paketToReceive.GetDataStringByName("V"); // verze
                if (strTemp != null) { sb.Append("Ver=").Append(strTemp).Append(" / "); }
                strTemp = paketToReceive.GetDataStringByName("R"); // revize
                if (strTemp != null) { sb.Append("Rev=").Append(strTemp).Append(" / "); }
                strTemp = paketToReceive.GetDataStringByName("U"); // Uid
                if (strTemp != null) { sb.Append("Uid=").Append(strTemp); }
                // ulozeni do seznamu 'Info(s)'
                infos.Add("SW: " + sb.ToString());
                sb.Clear();
            }
            // zjisteni cisla zarizeni (pokud nejake je nastaveno)
            paketToSend.CreateSimplePaket(ConstsPaket.Name.Cmd, "V", (byte)ConstsPaket.EnumCmd.Check);
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck /* | CommunExtAppConsts.EnumRecv.NoWaitToAnswer */) == ConstsCommun.EnumSendEndRecvStatus.OK)
            {
                int number = paketToReceive.GetDataNumberByName("N"); // pridelene cislo zarizeni
                if (number < int.MaxValue) { sb.Append("ID=").Append(number.ToString()); }
                // ulozeni do seznamu 'Info(s)'
                infos.Add("Ident: " + sb.ToString());
                sb.Clear();
            }

            // vystupni text nebude mit nakonci ' / ' (pro jistotu)
            string strOut = sb.ToString();
            tbDeviceInfo.Text = strOut.TrimEnd(' ', '/');
            // zobrazeni zakldniho 'Info'
            infosPos = 0;
            SelectInfo(tbDeviceInfo);
            return true;
        }

        // uzavreni / odpojeni zarizeni
        public static void DevClose()
        {
            CommunAppExt.Stop();
        }

        // --- ---

        // zobrazeni dalsiho 'Info'
        public static void SelectInfo(TextBlock tbDeviceInfo)
        {
            // pokud neni zadne 'Info', tak se nic nezobrazi
            if (infos.Count == 0) { tbDeviceInfo.Text = "---"; return; }
            // zobrazeni 'Info' na aktualni pozici
            tbDeviceInfo.Text = infos[infosPos];
            // posun pozice pro zobrazeni pristiho 'Info'
            if (++infosPos >= infos.Count) { infosPos = 0; }
        }

        // --- ---

        // nastaveni 'Ident' (musi to byt cislo)
        public static bool SetIdent(string text)
        {
            byte identNum = 0;
            // 'Ident' musi obsahovat pouze cislo (0-255)
            try { identNum = Byte.Parse(text); }
            catch
            {
                MessageBox.Show("Ident musi obsahovat pouze cisla a pismena!", "!!! Ident - Error !!!");
                return false;
            }

            paketToSend.BeginName(ConstsPaket.Name.Cmd); // paket typu 'Cmd'
            paketToSend.AddDataNameAndValues("V", (byte)ConstsPaket.EnumCmd.DevSet); // povel 'nastaveni zarizeni'
            paketToSend.AddDataNameAndValues("N", identNum); // nastaveni cisla / identu zarizeni
            paketToSend.End();
            // vyslat a prijmout odpoved
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck /* | CommunExtAppConsts.EnumRecv.NoWaitToAnswer */) == ConstsCommun.EnumSendEndRecvStatus.OK)
                { /* ... */ }
            return true;
        }

        // aktualizace SW
        public static void Update()
        {
            paketToSend.CreateSimplePaket(ConstsPaket.Name.Cmd, "V", (byte)ConstsPaket.EnumCmd.Update);
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck /* | CommunExtAppConsts.EnumRecv.NoWaitToAnswer */) == ConstsCommun.EnumSendEndRecvStatus.OK)
            { /* */ }
        }

        // konfigurace zobrazeni / komunikace (Shift)
        public static void Config()
        {
            if (Keyboard.Modifiers.HasFlag(ModifierKeys.Shift) == false)
            {
                // zobrazeni dialogu pro nastaveni zobrazeni
                UtilsSimpleGraph.ShowSettings(MainWindow);
            }
            else
            {
                // zobrazeni dialogu pro komunikaci (true -> zobrazeni)
                CommunAppExt.ShowDebug(GlobalConsts.AppNameStr, true,
                    // v zobrazeni povolen vstup do konfigurace
                    Commun.AppBase.MainWindow.EnumAppState.Started);
            }
        }

        // reset zarizeni
        public static void Reset(bool hard = false)
        {
            // nulovani poctu chyb
            errDataARMBufferCount = errDataARMTimeOverCount = errDataPCCount = 0;
            // Reset: SW / HW (Shift)
            paketToSend.BeginName(ConstsPaket.Name.Cmd);
            paketToSend.AddDataNameAndValues("V", (byte)ConstsPaket.EnumCmd.Reset);
            paketToSend.AddDataNameAndValues("X", (byte)((hard == false) ?
                ConstsPaket.EnumResetSubCmd.Soft : ConstsPaket.EnumResetSubCmd.Hard));
            paketToSend.End();
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck /* | CommunExtAppConsts.EnumRecv.NoWaitToAnswer */) == ConstsCommun.EnumSendEndRecvStatus.OK)
                { /* ... */ }
        }

        // vycteni konfigurace z 'Device'
        // OUT => A=address C = count => IN => A=address C = count D=datas
        // (vystup: nastavena presnost: null -> chyba / false -> 16bit / true -> 24bit )
        public static bool? AdsRegsToAppRegs()
        {
            bool? valueOut = null;
            // zadost o obsah registru z 'ADS'
            paketToSend.CreateSimplePaket(ConstsPaket.Name.Cmd, "V", (byte)ConstsPaket.EnumCmd.GetParams);
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive) == ConstsCommun.EnumSendEndRecvStatus.OK)
            {
                // adresa zacatku dat
                int address = paketToReceive.GetDataNumberByName("A");
                // pocet dat
                int count = paketToReceive.GetDataNumberByName("C");
                // zacatek dat
                paketToReceive.FindDataName("D");
               // postupne vycteni a ulozeni vsech dat
                for (int i = address; i < ((count <= registry.Count) ? count : registry.Count); i++)
                    { registry[i].ValueExt = (byte)paketToReceive.GetDataNumberNext(); }
                // pokud se predala i presnost, tak se pouzije
                if (count > registry.Count)
                {
                    // hodnota prenosti
                    byte valuePresnost = (byte)paketToReceive.GetDataNumberNext();
                    // vraceni stavu presnosti
                    valueOut = ((valuePresnost & 0x01) != 0x00);
                }
            }
            System.Threading.Thread.Sleep(100);
            // vraceni stavu presnosti (pokud se hodnota nastavila)
            return valueOut;
        }

        // zapis konfigurace do 'Device'
        // OUT => A=address C=count D=datas / IN => 
        public static void AppRegsToAdsRegs(bool bits16or24In)
        {
            byte from = 1; byte count = 25 /* + 1 */;
            paketToSend.BeginName(ConstsPaket.Name.Cmd);
            paketToSend.AddDataNameAndValues("V", (byte)ConstsPaket.EnumCmd.SetParams);
            paketToSend.AddDataNameAndValues("A", (byte)from);
            paketToSend.AddDataNameAndValues("C", (byte)(count + 1));
            paketToSend.AddDataName("D");
            // zapis vsech registru 'ADS'
            for (int i = from; i < (from + count); i++) { paketToSend.AddDataValues(registry[i].ValueExt); }
            // posledni hodnota je nastaveni 16/24bit dat (jde o zapis do konfigurace i kdyz to neni primo registr)
            paketToSend.AddDataValues((byte)((bits16or24In == false) ? 0 : 1));
            paketToSend.End();
            //int delka = paketToSend.Paket.Length;
            string a = paketToSend.Paket;
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck /* | CommunExtAppConsts.EnumRecv.NoWaitToAnswer */) == ConstsCommun.EnumSendEndRecvStatus.OK)
                { /* */ }
            System.Threading.Thread.Sleep(100);
        }

        // --- ---

        // adresar pro ulozeni nastavenych hodnot registru ADS
        private static string pathStoreRegPredef = System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory,
            "Setup", "RegistersPredef");

        // precteni nastavenych hodnot registru ADS
        public static void FileRegsToAppRegs()
        {
            // dialog pro vyber souboru
            OpenFileDialog openFileDialog = new OpenFileDialog() { InitialDirectory = pathStoreRegPredef };
            // dale pouze pri potvrzeni vyberu souboru
            if (openFileDialog.ShowDialog() == true)
            {
                // nacteni obsahu 'File XML' -> 'Class'
                StoreRegValues storeRegValues = XUtils.Xml.XmlSerialization.XmlFileToClass<StoreRegValues>(
                    pathStoreRegPredef, openFileDialog.FileName);
                // prepis nactenych hodnot do registru pro zobrazeni
                for (int i = 0; i < registry.Count; i++) { registry[i].ValueExt = storeRegValues.regs[i].Value; }
                // nazev souboru / konfigurace registru do zahlavi dialogu
                MainWindow.Title = GlobalConsts.AppNameStr + " ( Regs from file: " + System.IO.Path.GetFileNameWithoutExtension(openFileDialog.FileName) + " )";
            }
        }
        // ulozeni nastavenych hodnot registru ADS
        public static void AppRegsToFileRegs()
        {
            // trida pro ulozeni hodnot registru
            StoreRegValues storeRegValues = new StoreRegValues();
            // ulozeni aktualnich hodnot registru do teto tridy (pouze adresa - hodnota)
            for (int i = 0; i < registry.Count; i++)
            {
                storeRegValues.regs.Add(new StoreRegValue() { Address = registry[i].Address, Value = registry[i].ValueExt });
            }
            // dialog pro vyber souboru pro ulozeni
            SaveFileDialog saveFileDialog = new SaveFileDialog() { InitialDirectory = pathStoreRegPredef };
            // dale pouze pri potvrzeni nazvu souboru
            if (saveFileDialog.ShowDialog() == true)
            {
                // ulozeni hodnot registru do 'XML' souboru, 'Class' -> 'File XML'
                XUtils.Xml.XmlSerialization.ClassToXmlFile<StoreRegValues>(
                    pathStoreRegPredef, saveFileDialog.FileName + ".xml", storeRegValues, XUtils.Xml.XmlSerialization.EnumParams.Mini);
            }
        }

        // --- ---

        // setup ulozeni / (Shift) nulovani
        public static void SetupAction()
        {
            // setup ulozit / nulovat (Shift)
            paketToSend.BeginName(ConstsPaket.Name.Cmd);
            paketToSend.AddDataNameAndValues("V", (byte)ConstsPaket.EnumCmd.Setup);
            paketToSend.AddDataNameAndValues("X", (byte)((Keyboard.Modifiers.HasFlag(ModifierKeys.Shift) == false) ?
                ConstsPaket.EnumSetupSubCmd.Store : ConstsPaket.EnumSetupSubCmd.Clear));
            paketToSend.End();
            if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck /* | CommunExtAppConsts.EnumRecv.NoWaitToAnswer */) == ConstsCommun.EnumSendEndRecvStatus.OK)
            { /* ... */ }
        }

        // --- ---

        // jedno mereni do dialogu
        public static void GetOneValue()
        {
            // zadost o jeden odmer / jednu sadu dat
            if (Commun.AppExt.CommunAppExt.SendAndReceive(paketToSend.CreateSimplePaket(ConstsPaket.Name.Cmd, "V", (byte)ConstsPaket.EnumCmd.GetVlalue), out paketToReceive /*, CommunAppExtConsts.EnumRecv.NoIdentCheck */) == ConstsCommun.EnumSendEndRecvStatus.OK)
            {   // prijata data
                string data = paketToReceive.GetDataStringByName("D");
                // data jsou AsciiHex, vsech 27 vyctenych z ADS, data jsou vzdy 24bits

                // vystup bude retezec do 'MessageBox'
                StringBuilder sb = new StringBuilder();
                //sb.Append("Status ").Append(" - ").
                // vyzvednuti dat 'STATUS + CHANNELS'
                for (int i = 0; i < (1 + 8 /* actChannels */); i++)
                {
                    // pouze prvni polozka je 'Status', ostatni jsou 'Channel'
                    if (i == 0) { sb.Append("Status ").Append(" - "); }
                        else { sb.Append("Channel ").Append(i.ToString()).Append(" - "); }
                    // zobrazeni dat za nazvem
                    sb.Append(data.Substring(i * 6, 6));
                    sb.Append(" / ");
                    sb.Append(UInt32.Parse(data.Substring(i * 6, 6), System.Globalization.NumberStyles.AllowHexSpecifier));
                    sb.Append("\n");
                }
                // zobrazeni dialogu s textem
                MessageBox.Show(sb.ToString(), "One Value Set");
            }
        }

        // --- ---

        // spusteni kontinualniho mereni
        public static void MeasureStart()
        {
            // nulovani poctu chyb
            errDataARMBufferCount = errDataARMTimeOverCount = errDataPCCount = 0;
            UtilsSimpleGraph.titleTextExtra = null;
            DataGraphInit();
            // pokud 'Shift' -> pouze zobrazeni dialogu bez spusteni mereni / nulovani poctu chyb
            if ((Keyboard.Modifiers & ModifierKeys.Shift) == ModifierKeys.Shift) { return; }
            int speed = 0;
            #if ADS_VER_A
              paketToSend.BeginName(ConstsPaket.Name.Cmd);
              // povel 'Start'
              paketToSend.AddDataNameAndValues("V", (byte)ConstsPaket.EnumCmd.Start);
              // pocet kanalu (pouze cisla, kanaly pocinaje od prvniho)
              actChannels = 0; // nulovani poctu kanalu
              for (int i = 0; i < ChannelsTotal; i++)
              {
                // nastaveni povoleni / blokovani tohoto kanalu (registr pro kanaly jsou od daresy 5)
                UtilsSimpleGraph.items[i].Enable = ((registry[i + 5].ValueExt & 0x80) == 0x00);
                // pokud je kanal aktivni, tak se prida do poctu (ale samozrejme od zacatku / prvniho)
                if (UtilsSimpleGraph.items[i].Enable == true) { actChannels++; }
                // nastaveni rozliseni grafu podle aktualni presnosti mereni
                UtilsSimpleGraph.items[i].ValueMin = ((bits16or24 == false) ? GlobalConsts.ConstValue16bitsMin : GlobalConsts.ConstValue16bitsMax);
                UtilsSimpleGraph.items[i].ValueMin = ((bits16or24 == false) ? GlobalConsts.ConstValue24bitsMin : GlobalConsts.ConstValue24bitsMax);
            }
              // pokud neni zatrzen zadny kanal, tak se nastavi aspon jeden (ten prvni)
              if (actChannels == 0) { actChannels = 1; }
              paketToSend.AddDataNameAndValues("C", (byte)actChannels);
              // rozsah 16 nebo 24 bitu
              paketToSend.AddDataNameAndValues("P", (byte)((bits16or24 == false) ? 16 : 24));
              // rychlost mereni pomoci bitovych priznaku
              byte speedBits = (byte)(registry[1].ValueExt & 0x07); // pouze bit2, bit1, bit0
              // prevod bitovych priznaku na pocet vzorku za vterinu
              speed = 0;
              switch(speedBits)
              {
                case 0x00: { speed = 16000; break; } // 000 -> 16KHz
                case 0x01: { speed = 8000; break; } // 001 -> 8KHz
                case 0x02: { speed = 4000; break; } // 020 -> 4KHz
                case 0x03: { speed = 2000; break; } // 011 -> 2KHz
                case 0x04: { speed = 1000; break; } // 100 -> 1KHz
                case 0x05: { speed = 500; break; } // 101 -> 500Hz
                case 0x06: { speed = 250; break; } // 110 -> 250Hz
                                                   // 111 -> nepouzito
              }
              // zapis aktualni rychlosti do paketu (v poctu vzorku za vterinu)
              paketToSend.AddDataNameAndValues("S", speed);
              paketToSend.End();
            #endif
            #if ADS_VER_B
              // zadost o obsah registru z 'ADS'
              paketToSend.CreateSimplePaket(ConstsPaket.Name.Cmd, "V", (byte)GlobalConsts.EnumDeviceExtCmdsExt.MeasureParms);
              if (CommunAppExt.SendAndReceive(paketToSend, out paketToReceive) == ConstsCommun.EnumSendEndRecvStatus.OK)
              {
                // presnost (false -> 16bit / true -> 24bit)
                bits16or24 = (paketToReceive.GetDataNumberByName("P") == 1);
                // rychlost (cislo - pocet vzorku za vterinu)
                speed = paketToReceive.GetDataNumberByName("S");
                // pouzite kanaly (8bit cislo -> bitove priznaky pro kanaly)
                actChannels = paketToReceive.GetDataNumberByName("C");
                for (int i = 0; i < ChannelsTotal; i++)
                {
                    // nastaveni povoleni / blokovani tohoto kanalu (registr pro kanaly jsou od daresy 5)
                    // bit7 = 1 -> blokovan (false) / bit7 = 0 -> povolen (true)
                    UtilsSimpleGraph.items[i].Enable = ((actChannels & (1 << i)) != 0x00);
                    UtilsSimpleGraph.items[i].OnPropertyChanged("Enable");
                    // nastaveni rozliseni grafu podle aktualni presnosti mereni
                    UtilsSimpleGraph.items[i].ValueMin = ((bits16or24 == false) ? GlobalConsts.ConstValue16bitsMin : GlobalConsts.ConstValue16bitsMax);
                    UtilsSimpleGraph.items[i].ValueMin = ((bits16or24 == false) ? GlobalConsts.ConstValue24bitsMin : GlobalConsts.ConstValue24bitsMax);
                }
                // skutecne spusteni mereni
                paketToSend.CreateSimplePaket(ConstsPaket.Name.Cmd, "V", (byte)ConstsPaket.EnumCmd.Start);
              }
            #endif
            if (Commun.AppExt.CommunAppExt.SendAndReceive(paketToSend, out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck | ConstsCommun.EnumRecv.NoWaitToAnswer) == ConstsCommun.EnumSendEndRecvStatus.OK)
                { }
            // pocet vzorku na znacky 'ms'
            UtilsSimpleGraph.dlgShowData.speed = speed;
            UtilsSimpleGraph.dlgShowData.TimeMarkCalc();
        }

        // zastaveni kontinualniho mereni
        public static void MeasureStop()
        {
            //Commun.Connects.USBDevice.CntUSBDevice01.RecvBufferLowSizeUsed = 1024;
            if (CommunAppExt.SendAndReceive(paketToSend.CreateSimplePaket(ConstsPaket.Name.Cmd, "V", (byte)ConstsPaket.EnumCmd.Stop), out paketToReceive, ConstsCommun.EnumRecv.NoIdentCheck | ConstsCommun.EnumRecv.NoWaitToAnswer) == ConstsCommun.EnumSendEndRecvStatus.OK)
                { }
        }

        // pro mereni rychlost poctu vzorku za vterinu
        private static XUtils.Others.SpeedMeter speedMeter = new XUtils.Others.SpeedMeter() { CountNax = 1000 };
        private static string strTest = null;
        private static Int32 valueTemp = 0;
        private static int valueSumCount = 0;
        // prijem dat z kontinualniho mereni (paket X)
        private static void OnReceivePaketNameX(PaketToReceive paketX)
        {
            //strTest = paketX.GetPaketXData(0, 1 + 1 + 3 * 2 + 8 * 6 + 1 + 1); // TEST

            errDataARMBuffer = errDataARMTimeOver = errDataPC = false;

            // vyzvednuti dat z prijateho paketu
            try
            {
                int pos = 2; // pozice zacatku dat v paketu

                // vyzvednut 'Status' z ADS (na zacatku paketu)
                //statusADS = UInt32.Parse(paketX.GetPaketXData(pos, 6), System.Globalization.NumberStyles.AllowHexSpecifier);
                //pos += 6; // 'Status' ma 6 cislic ASCII-HEX

                // vyzvednuti hodnot z aktivnich kanalu
                for (int i = 0; i < ChannelsTotal; i++)
                {
                    // pokud tento kanal neni povolen, tak se preskoci
                    if (UtilsSimpleGraph.items[i].Enable == false) { continue; }

                    // podle poctu bitu hodnoty z ADS
                    if (bits16or24 == false)
                    {   // 16bits
                        valueTemp = Int32.Parse(paketX.GetPaketXData(pos, 4), System.Globalization.NumberStyles.AllowHexSpecifier);
                        if ((valueTemp & 0x00008000) == 0x00008000) { valueTemp = valueTemp - 0x00010000; }
                        pos += 4;
                    }
                    else
                    {   // 24 bits
                        valueTemp = Int32.Parse(paketX.GetPaketXData(pos, 6), System.Globalization.NumberStyles.AllowHexSpecifier);
                        if ((valueTemp & 0x00800000) == 0x00800000) { valueTemp = valueTemp - 0x01000000; }
                        pos += 6;
                    }
                    // ulozeni vyzvednute a zkorigovane hodnoty
                    UtilsSimpleGraph.items[i].value = valueTemp;

                    if (UtilsSimpleGraph.valueEvrAnable == true)
                    {
                        UtilsSimpleGraph.items[i].valueSum += valueTemp;
                        UtilsSimpleGraph.items[i].value -= UtilsSimpleGraph.items[i].valueEvr;
                    }
                }

                if (UtilsSimpleGraph.valueEvrAnable == true)
                {
                    valueSumCount++;
                    if (valueSumCount > UtilsSimpleGraph.valueEvrCount)
                    {
                        for (int i = 0; i < ChannelsTotal; i++)
                        {
                            UtilsSimpleGraph.items[i].valueEvr = (int)(UtilsSimpleGraph.items[i].valueSum / UtilsSimpleGraph.valueEvrCount);
                            UtilsSimpleGraph.items[i].valueSum = 0;
                        }
                        valueSumCount = 0;
                    }
                }

                // vyzvednuti 'Status' datove komunikace
                adsStatus = (byte)paketX.GetPaketXData(pos, 1)[0];
                // nastaveni chyby, pokud nejaka byla
                switch(adsStatus)
                {
                    case (byte)'1': { errDataARMBuffer = true; break; } // preteceni vysilaciho USB bufferu v ARM
                    case (byte)'2': { errDataARMTimeOver = true; break; } // nestihnuti vyzvednuti dat z ADS v ARM
                }
            }
            catch { errDataPC = true; }

            // !!! toto se musi udelat jinak, nejak dat do tridy grafu !!!

            // pokud nejaka chyba, tak se zobrazi
            if ((errDataARMBuffer == true) || (errDataARMTimeOver == true) || (errDataPC == true))
            {
                // pouze pokud je na co kreslit
                if (UtilsSimpleGraph.g != null)
                {
                    // pokud byla ztrata dat - preteceni bufferu v ARM (svisla barevna cara)
                    if (errDataARMBuffer == true)
                    {
                        errDataARMBufferCount++;
                        UtilsSimpleGraph.g.DrawLine(penErrARMBuffer, UtilsSimpleGraph.posActualX, 0, UtilsSimpleGraph.posActualX, UtilsSimpleGraph.dlgShowData.Height);
                    }
                    // pokud byla ztrata dat - nestihnuti vyzvednuti z ADS (svisla barevna cara)
                    if (errDataARMTimeOver == true)
                    {
                        errDataARMTimeOverCount++;
                        UtilsSimpleGraph.g.DrawLine(penErrARMTime, UtilsSimpleGraph.posActualX, 0, UtilsSimpleGraph.posActualX, UtilsSimpleGraph.dlgShowData.Height);
                    }
                    // pokud byla ztrata dat na PC vlyvem nechytnuti dat zacateku / konce paketu v PC (svisla barevna cara)
                    if (errDataPC == true)
                    {
                        errDataPCCount++;
                        UtilsSimpleGraph.g.DrawLine(penErrPC, UtilsSimpleGraph.posActualX, 0, UtilsSimpleGraph.posActualX, UtilsSimpleGraph.dlgShowData.Height);
                    }

                    // aktualizace pridavneho textu do zahlavi dialogu grafu
                    UtilsSimpleGraph.titleTextExtra =" [ ARM errBuffer = " + errDataARMBufferCount + " / ARM errTime = " + errDataARMTimeOverCount + " / PC err = " + errDataPCCount + " ]";
                }
            }

            // zobrazeni aktualnich dat (s preskakovanim vzorku)
            UtilsSimpleGraph.DataGraphValues();

            // ...
            if ((posVer != 0) || (UtilsSimpleGraph.dlgShowData.pos != 0))
            {
                UtilsSimpleGraph.g.DrawLine(UtilsSimpleGraph.dlgShowData.penValueLevel, 0, posVer, UtilsSimpleGraph.dlgShowData.Width, posVer);
                if ((UtilsSimpleGraph.dlgShowData.pos != 0) || (UtilsSimpleGraph.update == true))
                {
                    if (UtilsSimpleGraph.dlgShowData.pos != 0)
                    {
                        posVer = UtilsSimpleGraph.dlgShowData.pos;
                    }

                    int a = posVer - ((UtilsSimpleGraph.dlgShowData.Height - 50) / 2) - 32;

                    // nastaveni zobrazeni podle aktualniho rozsahu prevodniku
                    a = a * ((bits16or24 == false) ? 0x00007FFF : 0x007FFFFF ) / (UtilsSimpleGraph.dlgShowData.Height / 2);
                    a = (int)(a / UtilsSimpleGraph.opticalZoom);

                    Application.Current.Dispatcher.Invoke(() => { UtilsSimpleGraph.dlgShowData.ValueText(-a); });
                    UtilsSimpleGraph.dlgShowData.pos = 0;
                    UtilsSimpleGraph.update = false;
                }
            }
        }

        private static int posVer = 0;

        // --- data ---

        private static byte adsStatus = 0;
        // chyby
        private static bool errDataARMBuffer = false;
        private static bool errDataARMTimeOver = false;
        private static bool errDataPC = false;
        private static int errDataARMBufferCount = 0;
        private static int errDataARMTimeOverCount = 0;
        private static int errDataPCCount = 0;

        // barva pro horizontalni stredovou caru
        private static Pen penOsaX = new Pen(Color.Black, 1); // cerna
        private static Pen penErrARMBuffer = new Pen(Color.DarkGreen, 1); // tmave zelena
        private static Pen penErrARMTime = new Pen(Color.Red, 1); // cervena
        private static Pen penErrPC = new Pen(Color.Blue, 1); // modra

        // inicializace zobrazeni, volano jednou pred zobrazovanim
        private static void DataGraphInit()
        {
            UtilsSimpleGraph.DataGraphInit(MainWindow);
        }

        // --- ---

        public static void Close()
        {
            CommunAppExt.Stop();
        }
    }
}
