﻿using System;
using System.Windows;
using System.Windows.Input;

namespace ADSTester
{
    /// <summary>
    /// Interakční logika pro MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        // spusteni aplikace
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            AdsConfigV02.MainWindow = this;
            config.AutoGenerateColumns = false;
            config.CanUserAddRows = false;
            // pesnost mereni
            cbBits.ItemsSource = new String[] { "16", "24" }; cbBits.SelectedIndex = 0;
            bMeasuteStop.IsEnabled = false;
            AdsConfigV02.Init(config, "Default.xml");
        }

        // otevreni / pripojeni zarizeni
        private void OpenOnClick(object sender, RoutedEventArgs e)
        {
            bool done = AdsConfigV02.DevOpen(tbDeviceInfo);
            bDevClose.IsEnabled = tbDeviceInfo.IsEnabled = bInfo.IsEnabled = tbIdent.IsEnabled = bDevIdent.IsEnabled = bDevUpdate.IsEnabled = done;
            bRegs.IsEnabled = gControl.IsEnabled = done;
            bDevOpen.IsEnabled = !done;
            if (done == false) { tbDeviceInfo.Text = "---"; }
        }
        // uzavreni / odpojeni zarizeni
        private void CloseOnClick(object sender, RoutedEventArgs e)
        {
            AdsConfigV02.DevClose();
            bDevClose.IsEnabled = tbDeviceInfo.IsEnabled = bInfo.IsEnabled = tbIdent.IsEnabled = bDevIdent.IsEnabled = bDevUpdate.IsEnabled = false;
            bRegs.IsEnabled = gControl.IsEnabled = false;
            bDevOpen.IsEnabled = true;
            tbDeviceInfo.Text = "---";
        }

        // Info
        private void InfoOnClick(object sender, RoutedEventArgs e)
        {
            AdsConfigV02.SelectInfo(tbDeviceInfo);
        }

        // nastaveni identifikace HW
        private void IdentOnClick(object sender, RoutedEventArgs e)
        {
            // pokud byl zapis 'Ident' uspesny, tak se vymaze z dialogu
            if (AdsConfigV02.SetIdent(tbIdent.Text) == true) { tbIdent.Text = String.Empty; }
        }

        // aktualizace SW
        private void UpdateOnClick(object sender, RoutedEventArgs e)
        {
            // 0x1d50 0x604e Enp3 Enp1
            AdsConfigV02.Update();
            // uzavreni komunikace (zarizeni od odpoji)
            CloseOnClick(null, null);
        }

        // konfigurace zobrazeni / komunikace (Shift)
        private void CFGOnClick(object sender, RoutedEventArgs e)
        {
            // 0x1d50 0x604e Enp3 Enp1
            AdsConfigV02.Config();
        }

        // reset zarizeni soft / hard (Shift)
        private void ResetOnClick(object sender, RoutedEventArgs e)
        {
            bReset.IsEnabled = false;
            // 'SW Reset' / (Shift) 'HW Reset'
            AdsConfigV02.Reset((Keyboard.Modifiers.HasFlag(ModifierKeys.Shift) == true));
            System.Threading.Thread.Sleep(100);
            bReset.IsEnabled = true;

            // pri 'HW Reset' uzavreni komunikace (zarizeni od odpoji)
            if (Keyboard.Modifiers.HasFlag(ModifierKeys.Shift) == true)
                { CloseOnClick(null, null); }
        }

        // cteni obsahu registru z ADS / souboru (Shift)
        private void ReadRegsOnClick(object sender, RoutedEventArgs e)
        {
            bReadRegs.IsEnabled = false;
            if (Keyboard.Modifiers.HasFlag(ModifierKeys.Shift) == false)
            {   // cteni z ADS
                bool? state = AdsConfigV02.AdsRegsToAppRegs();
                // pokud je platny stav pro presnost, tak se pouzije (false -> 16bit / true -> 24bit)
                if (state != null) { cbBits.SelectedIndex = ((state == false) ? 0 : 1); }
            }
            // cteni ze souboru
            else { AdsConfigV02.FileRegsToAppRegs(); }
            System.Threading.Thread.Sleep(100);
            bReadRegs.IsEnabled = true;
        }

        // cteni obsahu registru do ADS / souboru (Shift)
        private void WriteRegsClick(object sender, RoutedEventArgs e)
        {
            bWriteRegs.IsEnabled = false;
            // zapis do ADS
            if (Keyboard.Modifiers.HasFlag(ModifierKeys.Shift) == false)
            {   // polozka 0 -> 16bit / polozka 1 -> 24bit
                AdsConfigV02.AppRegsToAdsRegs(cbBits.SelectedIndex == 1);
            }
            // zapis do souboru
            else { AdsConfigV02.AppRegsToFileRegs(); }
            System.Threading.Thread.Sleep(100);
            bWriteRegs.IsEnabled = true;
        }

        // setup ulozeni / (Shift) nulovani
        private void SetupActionOnClick(object sender, RoutedEventArgs e)
        {
            AdsConfigV02.SetupAction();
        }

        // vyzvednuti (pouze) jedne sady dat
        private void GetValueOnClick(object sender, RoutedEventArgs e)
        {
            bMeasute1X.IsEnabled = false;
            AdsConfigV02.GetOneValue();
            System.Threading.Thread.Sleep(100);
            bMeasute1X.IsEnabled = true;
        }

        // spusteni (opakovaneho) mereni
        private void MeasuteStartOnClick(object sender, RoutedEventArgs e)
        {
            AdsConfigV02.MeasureStart();
            bMeasuteStart.IsEnabled = false;
            bMeasuteStop.IsEnabled = true;
        }

        // zastaveni (opakovaneho) mereni
        private void MeasuteStopOnClick(object sender, RoutedEventArgs e)
        {
            AdsConfigV02.MeasureStop();
            bMeasuteStart.IsEnabled = true;
            bMeasuteStop.IsEnabled = false;
        }

        // ukonceni aplikace
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            AdsConfigV02.Close();
        }
    }
}
