#include "GlobalDef.h"
#include "usbd_cdc.h"

extern USBD_HandleTypeDef hUsbDeviceFS;
extern void Error_Handler();

// nformace o zarizeni (ulozeny ve FLASH)
const char InfoDevProd[] = INFO_DEVICE_PROD;
const char InfoDevName[] = INFO_DEVICE_NAME;
const char InfoHWType[] = INFO_HW_TYPE;
const char InfoHWVer[] = INFO_HW_VER;
const char InfoHWRev[] = INFO_HW_REV;
const char InfoSWType[] = INFO_SW_TYPE;
const char InfoSWVer[] = INFO_SW_VER;
const char InfoSWRev[] = INFO_SW_REV;
//const char InfoSWUid[] = INFO_SW_UID;
const uint8_t InfoSWUid[] = INFO_SW_UID;

#if defined(TEST_DATA)
  uint8_t testDataNumber = 0;   // vytvareni testovacich dat
#endif
#if defined(TEST_DATA_COUNT)
  uint32_t adsDataSendCount = 0;  // pocet odeslanych paketu / sad dat
#endif

GlobalConfig globalConfig;      // global konfigurace
uint8_t AdsConfig[27];          // konfigurace pro ADS

uint8_t actionFromCommun = ActFromCmnNone; // akce z komunikace

uint8_t adsChannels = 1;        // aktivni kanaly (bitove priznaky)
uint8_t adsBits16or24 = 0;      // zda se meri a prenasi 0 -> 16bit / 1 -> 24bit
int adsSpeed = 250;             // pocet vzorku za vterinu
uint8_t adsDataReady = 0;       // pocet detekovanych pulsu 'DataReady' z ADS
int16_t adsDataSendInfoOnOffLED = 0;       // pocet paketu pro blikani LED
int16_t adsDataLostCount = 0;   // pocet kontinualne zreacenych paketu (nezapsanych do USB bufferu)

uint32_t localFlags = 0;        // priznaky programu
// prenos dat
uint8_t dataSendStatus = PaketDataStatusOK;
int32_t dataSendErrNextTimeOnOffLED = 0;

uint8_t adsBuffer[MeasureBytes];

// buffer pro vysilani dat pres USB
uint8_t USB_BUFFER[USB_BUFFER_SIZE];
// ukazatel pro zapis do bufferu
uint32_t USBBufferInPtr = 0;
// ukazatel pro cteni z bufferu
uint32_t USBBufferOutPtr = 0;
// pro vypocet poctu hodnot / dat v bufferu
//uint32_t USBBufferDataLength = 0;

int paketLenght = 0;               // delka paketu
int USBBufferAddressMax = 0;       // maximalni adresa v bufferu
int USBBufferPacketsMax = 0;       // maximaln pocet paketu v bufferu
int USBBufferPacketsCount = 0;     // aktualni pocet paketu v bufferu

// --- ---

// konfigurace pro 'Setup'
SetupConfig setupConfig[] =
{
    { 1, sizeof(GlobalConfig), (void*)&globalConfig, 0, 0, 0 },
    { 2, sizeof(AdsConfig), (void*)&AdsConfig, 0, 1, 0 },
    { 0 }
};

// pole pro 'VrtAdr' pro 'Setup'
uint16_t VirtAddVarTab[(uint8_t)(sizeof(GlobalConfig) + sizeof(AdsConfig) - 1)];

// --- ---

void AdsDataToPacket();
void USBBufferIsFull();

// --- GPIOs ---

void MyInitGPIO()
{
    /* -1- Enable GPIO Clock (to be able to program the configuration registers) */
    __HAL_RCC_GPIOA_CLK_ENABLE();
    __HAL_RCC_GPIOB_CLK_ENABLE();

    GPIO_InitTypeDef  GPIO_InitStruct;
    GPIO_InitTypeDef  GPIO_InitStructIn;

    /* -2- Configure IO in output push-pull mode to drive external LEDs */
    GPIO_InitStruct.Mode  = GPIO_MODE_OUTPUT_PP;
    GPIO_InitStruct.Pull  = GPIO_PULLUP;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
    // LED 'Action'
    GPIO_InitStruct.Pin = LED_ACTION_PIN;
    HAL_GPIO_Init(LED_ACTION_PORT, &GPIO_InitStruct);
    // LED 'Error'
    GPIO_InitStruct.Pin = LED_ERROR_PIN;
    HAL_GPIO_Init(LED_ERROR_PORT, &GPIO_InitStruct);
    // Sleep
    GPIO_InitStruct.Pin = ADS_SLEEP_PIN;
    HAL_GPIO_Init(ADS_SLEEP_PORT, &GPIO_InitStruct);
    // SPI-CS
    GPIO_InitStruct.Pin = SPI_CS_PIN;
    HAL_GPIO_Init(SPI_CS_PORT, &GPIO_InitStruct);
    // Start
    GPIO_InitStruct.Pin = ADS_START_PIN;
    HAL_GPIO_Init(ADS_START_PORT, &GPIO_InitStruct);
    // Reset
    GPIO_InitStruct.Pin = ADS_RESET_PIN;
    HAL_GPIO_Init(ADS_RESET_PORT, &GPIO_InitStruct);

    GPIO_InitStructIn.Pin   = ADS_READY_PIN;
    GPIO_InitStructIn.Mode  = GPIO_MODE_INPUT;
    GPIO_InitStructIn.Mode  = GPIO_MODE_IT_FALLING;
    //GPIO_InitStructIn.Pull  = GPIO_PULLUP;
    GPIO_InitStructIn.Speed = GPIO_SPEED_FAST;
    HAL_GPIO_Init(ADS_READY_PORT, &GPIO_InitStructIn);

    // preruseni od ADS_READY_PIN (EXTI1)
    HAL_NVIC_SetPriority(EXTI1_IRQn, 2, 0);
    HAL_NVIC_EnableIRQ(EXTI1_IRQn);
}

void EXTI1_IRQHandler(void)
{
    HAL_GPIO_EXTI_IRQHandler(ADS_READY_PIN);
    /* Make sure that interrupt flag is set */
    //if (EXTI_GetITStatus(EXTI_Line1) != RESET)
    //{
    //}

    #if defined (TEST_BY_TIMER)
    #else
        // priznak dat se nastavuje pouze pokud je kontinualni mereni
        // (dalsi data jsou v ADS pro vycteni)
        if(localFlags & LocFlagADSDataContinuous) { adsDataReady++; }
    #endif
}

// --- TIMs ---

// casovace pro simulaci mereni
#if defined(TEST_BY_TIMER)
TIM_HandleTypeDef TimHandle;
void MyTestTimerInit(int value)
{
    __HAL_RCC_TIM2_CLK_ENABLE();

/* Compute the prescaler value to have TIMx counter clock equal to 10000 Hz */
// SystemCoreClock = 80Mhz
  //uint32_t uwPrescalerValue = (uint32_t)(SystemCoreClock / 10000) - 1;

  /* Set TIMx instance */
  TimHandle.Instance = TIM2;

  /* Initialize TIMx peripheral as follows:
       + Period = 10000 - 1
       + Prescaler = (SystemCoreClock/10000) - 1
       + ClockDivision = 0
       + Counter direction = Up
  */
  TimHandle.Init.Period            = value;
  TimHandle.Init.Prescaler         = 80; // delicka na 1us
  TimHandle.Init.ClockDivision     = 0;
  TimHandle.Init.CounterMode       = TIM_COUNTERMODE_UP;
  TimHandle.Init.RepetitionCounter = 0;

  HAL_NVIC_SetPriority(TIM2_IRQn, 4, 0);
  HAL_NVIC_EnableIRQ(TIM2_IRQn);

  if (HAL_TIM_Base_Init(&TimHandle) != HAL_OK)
    /* Initialization Error */
    { Error_Handler(); }
}

void MyTestTimerStart()
{
      /*##-2- Start the TIM Base generation in interrupt mode ####################*/
  /* Start Channel1 */
  if (HAL_TIM_Base_Start_IT(&TimHandle) != HAL_OK)
    /* Starting Error */
    { Error_Handler(); }
}

// zastaveni casovace
void MyTestTimerStop()
{
    //HAL_TIM_Base_Stop_IT(&TimHandle)
    if (HAL_TIM_Base_Stop_IT(&TimHandle) != HAL_OK)
        /* Starting Error */
        { Error_Handler(); }
}

void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
  // preruseni se obsluhuje
    //HAL_TIM_ TIM_ClearITPendingBit(TIM16, TIM_IT_Update);
    // priznak dat se nastavuje pouze pokud je kontinualni mereni
    // (dalsi data jsou v ADS pro vycteni)
    if(localFlags & LocFlagADSDataContinuous) { adsDataReady++; }
}

#endif

// --- SPI ---

SPI_HandleTypeDef SpiHandle;
void MyInitSPI()
{
//    /*##-1- Configure the SPI peripheral #######################################*/
//  /* Set the SPI parameters */
    SpiHandle.Instance               = SPI1;
    SpiHandle.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_8;
    SpiHandle.Init.Direction         = SPI_DIRECTION_2LINES;
    SpiHandle.Init.CLKPhase          = SPI_PHASE_2EDGE;
    SpiHandle.Init.CLKPolarity       = SPI_POLARITY_LOW;
    SpiHandle.Init.DataSize          = SPI_DATASIZE_8BIT;
    SpiHandle.Init.FirstBit          = SPI_FIRSTBIT_MSB;
    SpiHandle.Init.TIMode            = SPI_TIMODE_DISABLE;
    SpiHandle.Init.CRCCalculation    = SPI_CRCCALCULATION_DISABLE;
    SpiHandle.Init.CRCPolynomial     = 7;
    SpiHandle.Init.CRCLength         = SPI_CRC_LENGTH_DATASIZE;
    SpiHandle.Init.NSS               = SPI_NSS_SOFT;
    SpiHandle.Init.NSSPMode          = SPI_NSS_PULSE_DISABLE;
    SpiHandle.Init.Mode = SPI_MODE_MASTER;

    GPIO_InitTypeDef  GPIO_InitStructSPI;

    GPIO_InitStructSPI.Pin = GPIO_PIN_5|/*GPIO_PIN_6|*/GPIO_PIN_7;
    GPIO_InitStructSPI.Mode = GPIO_MODE_AF_PP;
    GPIO_InitStructSPI.Pull = GPIO_NOPULL; //GPIO_PULLUP;
    GPIO_InitStructSPI.Speed = GPIO_SPEED_HIGH;
    GPIO_InitStructSPI.Alternate = GPIO_AF5_SPI1;
    HAL_GPIO_Init(GPIOA, &GPIO_InitStructSPI);

    GPIO_InitStructSPI.Pin = GPIO_PIN_6;
    GPIO_InitStructSPI.Pull = GPIO_PULLUP;
    HAL_GPIO_Init(GPIOA, &GPIO_InitStructSPI);

    __HAL_RCC_SPI1_CLK_ENABLE();

    if(HAL_SPI_Init(&SpiHandle) != HAL_OK)
    {
        /* Initialization Error */
        Error_Handler();
    }
}

void SPI_CS_HIGH() { HAL_GPIO_WritePin(SPI_CS_PORT, SPI_CS_PIN, GPIO_PIN_SET); }
void SPI_CS_LOW() { HAL_GPIO_WritePin(SPI_CS_PORT, SPI_CS_PIN, GPIO_PIN_RESET); }

uint8_t dataOutX = 0;
uint8_t aaa = 0;
// soucasne vyslani a prijem jedne 8bit hodnoty
uint8_t SPI_SEND_RECEIVE_8BIT(uint8_t dataIn, uint8_t action)
{
    dataOutX = 0;
    // pkud je zacatek prensu
    if(action & SPI_CS_START) { SPI_CS_LOW(); HAL_Delay(1); }

    //HAL_SPI_StateTypeDef aa = HAL_SPI_GetState(&SpiHandle);

    // vycteni predchoziho prijmu (aby byl vstupni buffer / fronta prazdne)
    //while (SPI_GetReceptionFIFOStatus(SPI1) != SPI_ReceptionFIFOStatus_Empty)
    //    { SPI_ReceiveData8(SPI1); }
    // cekani na ukonceni preseleho vysilani (pokud nejake bylo)
    //while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET); //wait buffer empty
    //while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_BSY) == SET); //wait finish sending
    // vyslani pozadovane hodnoty

    //while(1)
    //{
    //if(HAL_SPI_Transmit(&SpiHandle, &dataIn, 1, 1000) != HAL_OK)
    //{
    //    aaa = 1;
    //}

    if(HAL_SPI_TransmitReceive(&SpiHandle, &dataIn, &dataOutX, 1, 1000) != HAL_OK)
    {
        aaa = 1;
    }

    //HAL_Delay(5);
    //}

    //HAL_SPI_StateTypeDef bb = HAL_SPI_GetState(&SpiHandle);

    //HAL_Delay(10);

    //SPI_SendData8(SPI1, dataIn);
    // cekani na ukonceni skutecneho vysilani
    //while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_TXE) == RESET); //wait buffer empty
    //while (SPI_I2S_GetFlagStatus(SPI1, SPI_I2S_FLAG_BSY) == SET); //wait finish sending
    // cekani na dokonceni prijmu (vzdy nejaky prijem je)
    //while (!(SPI1->SR & SPI_I2S_FLAG_RXNE));
    // vraceni prijate hodnoty (treba se zahodi)
    //uint8_t dataOutX = SPI_ReceiveData8(SPI1);
    //dataOutX = 0;
    //if(HAL_SPI_Receive(&SpiHandle, &dataOutX, 1, 1000) != HAL_OK)
    //{
    //    aaa = 2;
    //}

    //HAL_SPI_StateTypeDef cc = HAL_SPI_GetState(&SpiHandle);

    // pokud je konec prenosu
    if(action & SPI_CS_END) { HAL_Delay(1); SPI_CS_HIGH(); HAL_Delay(1); }
    // vraceni prijate hodnoty
    return dataOutX;

    //return 0;
}

uint32_t spixbase = 0x00;

// --- ADS ---

void ADS_RESET_PIN_SET(GPIO_PinState state) { HAL_GPIO_WritePin(ADS_RESET_PORT, ADS_RESET_PIN, state); }
void ADS_SLEEP_PIN_SET(GPIO_PinState state) { HAL_GPIO_WritePin(ADS_SLEEP_PORT, ADS_SLEEP_PIN, state); }
void ADS_START_PIN_SET(GPIO_PinState state) { HAL_GPIO_WritePin(ADS_START_PORT, ADS_START_PIN, state); }
// ADS_READY_PIN
uint8_t ADS_READY_PIN_GET() { HAL_GPIO_ReadPin(ADS_READY_PORT, ADS_READY_PIN); }

// --- ---

// modra LED (akce)
void ActionOnLED() { HAL_GPIO_WritePin(LED_ACTION_PORT, LED_ACTION_PIN, GPIO_PIN_SET); }
void ActionOffLED() { HAL_GPIO_WritePin(LED_ACTION_PORT, LED_ACTION_PIN, GPIO_PIN_RESET); }
void ActionToggleLED() { HAL_GPIO_TogglePin(LED_ACTION_PORT, LED_ACTION_PIN); }
// cervena LED (error)
void ErrorOnLED() { HAL_GPIO_WritePin(LED_ERROR_PORT, LED_ERROR_PIN, GPIO_PIN_SET); }
void ErrorOffLED() { HAL_GPIO_WritePin(LED_ERROR_PORT, LED_ERROR_PIN, GPIO_PIN_RESET); }
void ErrorToggleLED() { HAL_GPIO_TogglePin(LED_ERROR_PORT, LED_ERROR_PIN); }

// --- ---

// prvotni inicializace pred 'Loop'
void MyInit()
{
    localFlags = 0;
    adsDataReady = 0;
    adsDataLostCount = 0;

    MyComInit();        // inicializae komunikace
    MyInitGPIO();

    SPI_CS_HIGH();

    MyInitSPI();
    //MyTestTimerInit();

    ADS_SLEEP_PIN_SET(GPIO_PIN_SET); // GPIO_SetBits(GPIOA, GPIO_Pin_2); // SLEEP
    HAL_Delay(10);
    ADS_START_PIN_SET(GPIO_PIN_RESET); // GPIO_ResetBits(GPIOB, GPIO_Pin_2); // START
    HAL_Delay(10);
    ADS_RESET_PIN_SET(GPIO_PIN_SET); // GPIO_SetBits(GPIOB, GPIO_Pin_1); // RESET
    HAL_Delay(10);
    ADS_RESET_PIN_SET(GPIO_PIN_RESET); // GPIO_ResetBits(GPIOB, GPIO_Pin_1);
    HAL_Delay(10);
    ADS_RESET_PIN_SET(GPIO_PIN_SET); //GPIO_SetBits(GPIOB, GPIO_Pin_1);

    // inicializace SPI pomoci impulsu na /CS
    HAL_Delay(5);
    SPI_CS_LOW();
    HAL_Delay(5);
    SPI_CS_HIGH();
    HAL_Delay(500);

    // 11h -> Data Stop
    SPI_SEND_RECEIVE_8BIT(0x11, SPI_CS_START | SPI_CS_END);

    // cislo zarizeni neni (zatim) nastaveno
    globalConfig.DevNum = 0;

    // cteni cele konfigurace (test zda je ADS pritoment)
    AdsChipToAdsConfig();
    // vychozi preciznost je '16bit'
    AdsConfig[ADSRegAdrEXT1] = 0x00;

    // kontrola identifikace ADS 'ADS1298 = 0x92'
    if (AdsConfig[ADSRegAdrID] != 0x92)
    {
        // chyba
    }

    SetupInit();            // inicializace 'Setup'
    SetupLoad(NULL);        // macteni ulozene konfigurace
    AdsConfigToAdsChip();   // jeji zapis do ADS (pro pozdejsi cteni)

    // blikani po zapnuti napajeni
    for(int i = 0; i < 5; i++)
    {
        ActionOnLED(); //ErrorOnLED();
        HAL_Delay(400);
        ActionOffLED(); //ErrorOffLED();
        HAL_Delay(400);
    }
}

// citac systemovych tiku 1ms
extern __IO uint32_t uwTick;
// stav / pocet tiku pro zhlasnuti 'Action' LED po vykonani povelu
uint32_t uwTickToActionEnd = 0;
// stav / pocet tiku pro zmenu 'Error' LED pri chybe na USB
uint32_t uwTickToErrorToggle = 0;

uint8_t paketDone = 0;

// volano z hlavniho 'Loop'
void MyLoop()
{
    // --- spusteni kontinualniho mereni ---

    // pokud je pozadavek na spusteni kontinualniho mereni
    if (localFlags & LocFlagRunningAsk)
    {
        // adresa pro zapis / cteni SPI (zrychleni)
        spixbase = (uint32_t)SPI1 + 0x0C;

        #if defined(TEST_DATA_COUNT)
            adsDataSendCount = 0;  // pocet odeslanych paketu / sad dat
        #endif

        // pro jistotu se nejaky cas pocka
        HAL_Delay(1000);
        // pozadavek na spusteni behu vykonan
        localFlags &= ~LocFlagRunningAsk;
        // beh spusten
        localFlags |= LocFlagRunning;
        // sviti LEDs
        ActionOnLED();
        // test generovani paketu / dat pomoci casovace
        #if defined(TEST_BY_TIMER)
          // spusteni casovace pro simulaci dat
          MyTestTimerStart();
        #else
          // povel pro kontinualni mereni
          SPI_SEND_RECEIVE_8BIT(0x10, SPI_CS_START | SPI_CS_END);
          // skutecne spusteni kontinualniho mereni
          ADS_START_PIN_SET(GPIO_PIN_SET); // START
          // zacatek komunikace na SPI
          SPI_CS_LOW();
          //// vycteni dosud obsahu
          //while (SPI_GetReceptionFIFOStatus(SPI1) != SPI_ReceptionFIFOStatus_Empty)
          //    { SPI_ReceiveData8(SPI1); }
        #endif
        // nulovani pozadavku na odber dat
        adsDataReady = 0;
        // zmena stavul LED bude po 1s prenosu
        adsDataSendInfoOnOffLED = adsSpeed;
        // pro povoleni generovani pozadavku na vycteni dat
        localFlags |= LocFlagADSDataContinuous;
    }

    // --- vyzvednuti namerenych dat ---

    // pokud jsou nejaka data z ADS, tak se musi vyzvednout
    // (pri kontinualnim mereni)
    if (adsDataReady > 0)
    {
        // pokud se nejaka data nestihla -> chyba
        if (adsDataReady > 1)
        {   // nastaveni priznaku nestihani cteni dat z ADS
            dataSendStatus = PaketDataStatusTimeOver;
        }

        // data se vycitaji / zpracovavaji
        adsDataReady = 0;
        // zakazani preruseni
        __disable_irq();
        // presun dat 'ADS' -> 'USB'
        AdsDataToPacket();
        // povoleni preruseni
        __enable_irq();

        // pokud se nejaka data nestihla -> chyba
        if (adsDataReady > 0)
        {   // nastaveni priznaku nestihani cteni dat z ADS
            dataSendStatus = PaketDataStatusTimeOver;
        }

        // jedny data byla prenesena
        adsDataSendInfoOnOffLED--;
        // pri preneseni poctu dat za dobu 1s
        if (adsDataSendInfoOnOffLED == 0)
        {
            // se zmeni stav LED 'Action'
            ActionToggleLED();
            // dalsi zmena LED bude zase az po 1s mereni
            adsDataSendInfoOnOffLED = adsSpeed;
        }
    }

    // --- signalizace presuseni prenosu dat pres USB ---

    // preruseni komunikace pres USB (dlouhodobe bez automaticke obnovy)
    if (localFlags & LocFlagDataSendErr)
    {
        // zda uplynul cas pro zmenu stavu LED
        if (uwTick > uwTickToErrorToggle)
        {
            ErrorToggleLED();
            // interval do pristi zmeny
            uwTickToErrorToggle = uwTick + BlickTimeOnUSBDataStop;
        }
    }

    // --- odeslani dat z bufferu do USB ---

    // pokud je bufferu minimalni pocet dat tak se musi odeslat
    //if ((USBBufferPacketsCount * paketLenght) >= USB_SEND_BLOCK)
    if ((USBBufferPacketsCount * paketLenght) > USB_SEND_BLOCK)
        { SendDataToUSB(); }

    // --- zpracovani prijatych paketu ---

    // test na prijem celeho paketu
    paketDone = MyComPaketProcess();

    // pokud je akce vyzadana z komunikace az po odeslani odpovedi
    if (actionFromCommun != ActFromCmnNone)
    {
        // SW Reset aplikace
        if (actionFromCommun == ActFromCmnResetAppSW)
            { HAL_Delay(2000); MyInit(); }
        // HW Reset aplikace
        else if (actionFromCommun == ActFromCmnResetAppHW)
            { HAL_Delay(5000); XUTILS_Reset(ResetTypeApp); }
        // HW Reset do 'BootLoader'
        else if (actionFromCommun == ActFromCmnResetBootHW)
            { HAL_Delay(5000); XUTILS_Reset(ResetTypeBoot); }
        // pozadovana akce vykonana
        actionFromCommun = ActFromCmnNone;
    }

    // pokud byl prijat cely paket a zpracovan, tak ...
    if (paketDone == 1)
    {   // rozsviti se na 200ms ActionLED
        uwTickToActionEnd = uwTick + 200;
        ActionOnLED();
    }

    // pokud je cas pro rozsviceni LED
    if (uwTickToActionEnd != 0)
    {   // a cas dobehl do konce
        if(uwTick > uwTickToActionEnd)
        {   // tak se LED zhasne
            ActionOffLED();
            uwTickToActionEnd = 0;
        }
    }
}

// --- ---

// AdsConfig[] -> obvod
void AdsConfigToAdsChip()
{
    // zahaji zapisu registru
    SPI_SEND_RECEIVE_8BIT(0x40 | ADSRegAdrWriteStart, SPI_CS_START);
    // pocet hodnot pro zapis o jednu mensi
    SPI_SEND_RECEIVE_8BIT((ADSRegAdrWriteEnd - ADSRegAdrWriteStart /* - 1 */), SPI_CS_NONE);

    uint8_t dataTemp = 0;
    // postupne vyzvednuti vsech dat pro zapis
    for(int i = ADSRegAdrWriteStart; i <= ADSRegAdrWriteEnd; i++)
    {
        // vyzvednuti hodnoty z konfigurace
        dataTemp = AdsConfig[i];
        // jeji zapis do obvodu
        SPI_SEND_RECEIVE_8BIT(dataTemp, SPI_CS_NONE);
    }
    // konec komunikace na SPI
    SPI_CS_HIGH();
}

// obvod -> AdsConfig[]
void AdsChipToAdsConfig()
{
    // zahajeni cteni registru (od registru adresy '0')
    SPI_SEND_RECEIVE_8BIT(0x20, SPI_CS_START);
    // pocet hodnot pro cteni o jednu mensi
    SPI_SEND_RECEIVE_8BIT((ADSRegAdrReadEnd - ADSRegAdrReadStart /* - 1 */), SPI_CS_NONE);

    uint8_t dataTemp = 0;
    // precteni vsech hodnot pro odeslani zpet
    for(int i = ADSRegAdrReadStart; i <= ADSRegAdrReadEnd; i++)
    {
        // precteni jedne hodnoty
        dataTemp = SPI_SEND_RECEIVE_8BIT(0x00, SPI_CS_NONE);
        // jeji zapis do konfigurace
        AdsConfig[i] = dataTemp;
    }
    // konec komunikace na SPI
    SPI_CS_HIGH();
}

// stanoveni nekterych hodnot pro mereni z konfigurace 'AdsConfig[]'
void AdsConfigToMeasureParams()
{
    // nastaveni pouzite presnosti (ADSRegAdrEXT1 - bit0)
    adsBits16or24 = AdsConfig[ADSRegAdrEXT1] & 0x01;

    // stanoveni pouzitych kanalu bitove
    adsChannels = 0;
    // (a) stanoveni delky vysilaneho paketu
    paketLenght = 1 + 1; // PAKET_START + 'X'
    uint8_t dataTemp = 0;
    // projiti vsech kanalu
    for(int i = 0; i < 8; i++)
    {
        // vyzvednuti konfigurace jednoho kanalu
        dataTemp = AdsConfig[ADSRegAdrCH1SET + i];
        // pokud je kanal pouzit (bit7 = 0)
        if ((dataTemp & 0x80) == 0x00)
        {
            // tak se da log1 do odpovidajiciho bitu (podle cisla kanalu)
            adsChannels |= (1 << i);
            // 16bit hodnota je 4 znaky / 24bit hodnota je 6 znaku
            paketLenght += ((adsBits16or24 == 0) ? 4 : 6);
        }
    }
    paketLenght += (1 + 1); // 'Status' + PAKET_END

    // stanoveni vzorkovaci rychlosti
    // ADSRegAdrCON1 - bit2(DR2), bit1(DR1), bit0(DR0)
    dataTemp = AdsConfig[ADSRegAdrCON1] & 0x07;
    switch(dataTemp)
    {
        case 0x00: { adsSpeed = 16000; break; }    // 000 -> 16KHz
        case 0x01: { adsSpeed = 8000; break; }     // 001 -> 8KHz
        case 0x02: { adsSpeed = 4000; break; }     // 010 -> 4KHz
        case 0x03: { adsSpeed = 2000; break; }     // 011 -> 2KHz
        case 0x04: { adsSpeed = 1000; break; }     // 100 -> 1KHz
        case 0x05: { adsSpeed = 500; break; }      // 101 -> 500Hz
        case 0x06: { adsSpeed = 250; break; }      // 110 -> 250Hz
        default: { adsSpeed = 250; break;}
    }
    // pokud ADSRegAdrCON1 - bit7(HR) = 1 -> speed 2x
    if (AdsConfig[ADSRegAdrCON1] & 0x80) { adsSpeed *= 2; }

    // pocet celych paketu, kterych lze vlozit do bufferu
    USBBufferPacketsMax = USB_BUFFER_SIZE / paketLenght;
    // konecna adresa pri vlozeni maximlaniho poctu paketu
    USBBufferAddressMax = USBBufferPacketsMax * paketLenght;
    // zatim neni v bufferu zadny paket
    USBBufferPacketsCount = 0;
}

// --- ---

void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
}

void SendDataToUSB()
{
    USBD_CDC_HandleTypeDef *hcdc = (USBD_CDC_HandleTypeDef*)hUsbDeviceFS.pClassData;
    // pouze pokud je mozno vysilat
    if (hcdc->TxState == 0)
    {
        // pocet dat pro preneseni
        int pocetBytes = USBBufferPacketsCount * paketLenght;
        // pocet nesmi byt vetsi nez povoleny block
        if (pocetBytes > USB_SEND_BLOCK)
        {
            pocetBytes = (USB_SEND_BLOCK / paketLenght) * paketLenght;
        }
        // pokud je to do konce bufferu, tak se vysle jen to do konce
        if (pocetBytes > (USBBufferAddressMax - USBBufferOutPtr))
        {
            pocetBytes = (USBBufferAddressMax - USBBufferOutPtr);
        }

        // vyslani bloku dat
        MyComPaketSend(USB_BUFFER + USBBufferOutPtr, pocetBytes);
        // tolik dat bylo prave odeslano
        USBBufferPacketsCount -= (pocetBytes / paketLenght);
        // o tolik se posune ukazatel cteni v bufferu
        USBBufferOutPtr += pocetBytes;
        // pokud je ukazatel na konci, tak se presuna opet na zacatek
        if (USBBufferOutPtr == USBBufferAddressMax)
            { USBBufferOutPtr = 0; }
    }
}

// --- prenos dat z ADS do USB bufferu ---

uint8_t spiData = 0;        // data ctena z SPI
int yesChannel = 0;         // zda jde o aktivni kanal
void AdsDataToPacket()
{
    // mereni casu pro sestaveni paketu ze sady dat
    #if defined(TEST_TIME_TO_SEND)
      int testTimeCreatePaket = TIM_GetCounter(TIM16);
    #endif

    // test zda je ve vysilacim bufferu volno
    if (USBBufferPacketsCount >= USBBufferPacketsMax)
        // pokud je plny, tak konec
        { USBBufferIsFull(); return; }

    // vycteni predchoziho prijmu (aby byl vstupni buffer / fronta prazdne)
    //while (SPI_GetReceptionFIFOStatus(SPI1) != SPI_ReceptionFIFOStatus_Empty)
    //    { SPI_ReceiveData8(SPI1); }

    // aktualni pozice / ukazatel pri zapisu do vystupniho USB bufferu
    uint8_t *buffer_write_ptr = USB_BUFFER + USBBufferInPtr;

    // vlozeni zacatku paketu ('start znak' + 'typ paketu')
    *buffer_write_ptr++ = PAKET_START;
    *buffer_write_ptr++ = 'X';
    //if (buffer_write_ptr == buffer_write_ptr_max) { buffer_write_ptr = USB_BUFFER; }

    // vycteni 'Status' -> zahodi se (zatim)
    for(int i = 0; i < 3; i++)
    {
        // zacne se prijimat dalsi hodnota (aby se nezdrzovalo)
        *(__IO uint8_t *) spixbase = 0x00;
        // cekani na dokonceni prijmu na SPI
        while (!(SPI1->SR & SPI_FLAG_RXNE));
        // vyzvednuti hodnoty prijate z SPI
        spiData = *(__IO uint8_t *) spixbase;
    }

    // postupne vycteni vsech 8 kanalu a vlozeni dat do paketu
    for(int i = 0; i < 8; i++)
    {
        // zahajeni prijmu dat na SPI (byte0)
        *(__IO uint8_t *) spixbase = 0x00;

        // zda je tento kanal aktivni, vysila se do PC
        yesChannel = (adsChannels & (1 << i));

        // cekani na dokonceni prijmu na SPI (byte0)
        while (!(SPI1->SR & SPI_FLAG_RXNE));
        // vyzvednuti hodnoty prijate z SPI (byte0)
        spiData = *(__IO uint8_t *) spixbase;
        // pokud jsou testovaci data
        #if defined(TEST_DATA)
          spiData = testDataNumber;
        #endif

        // zacne se prijimat dalsi hodnota, aby se nezdrzovalo (byte1)
        *(__IO uint8_t *) spixbase = 0x00;

        if (yesChannel) // pouze aktivni kanal
        {
            // ulozeni jako dve ASCII-HEX cisla (byte0)
            *buffer_write_ptr++ = bin2hex[(spiData >> 4) & 0x0F];
            *buffer_write_ptr++ = bin2hex[spiData & 0x0F];
        }

        // cekani na dokonceni prijmu na SPI (byte1)
        while (!(SPI1->SR & SPI_FLAG_RXNE));
        // vyzvednuti hodnoty prijate z SPI (byte1)
        spiData = *(__IO uint8_t *) spixbase;
        // pokud jsou testovaci data
        #if defined(TEST_DATA)
          spiData = testDataNumber;
        #endif

        // zacne se prijimat dalsi hodnota, aby se nezdrzovalo (byte2)
        *(__IO uint8_t *) spixbase = 0x00;

        if (yesChannel) // pouze aktivni kanal
        {
            // ulozeni jako dve ASCII-HEX cisla (byte1)
            *buffer_write_ptr++ = bin2hex[(spiData >> 4) & 0x0F];
            *buffer_write_ptr++ = bin2hex[spiData & 0x0F];
        }

        // cekani na dokonceni prijmu na SPI (byte2)
        while (!(SPI1->SR & SPI_FLAG_RXNE));
        // vyzvednuti hodnoty prijate z SPI (byte2)
        spiData = *(__IO uint8_t *) spixbase;
        // pokud jsou testovaci data
        #if defined(TEST_DATA)
          spiData = testDataNumber;
        #endif

        if ((yesChannel) && (adsBits16or24 == 1)) // pouze aktivni kanal a 24bit
        {
            // ulozeni jako dve ASCII-HEX cisla (byte2)
            *buffer_write_ptr++ = bin2hex[(spiData >> 4) & 0x0F];
            *buffer_write_ptr++ = bin2hex[spiData & 0x0F];
        }
    }

    // --- ---

    /*
    // zahajeni prijmu dat na SPI (prvni datova hodnota)
    *(__IO uint8_t *) spixbase = 0x00;

    uint8_t hodnot = (adsChannels * 3) - 1;
    int v = 0; // zda se resi prvni, druha, treti hodnota z kanalu
    // postupne vycteni vsech 8 kanalu a vlozeni dat do paketu
    for(int i = 0; i < hodnot/; i++)
    {
        // cekani na dokonceni prijmu na SPI
        while (!(SPI1->SR & SPI_FLAG_RXNE));
        // vyzvednuti hodnoty prijate z SPI
        spiData = *(__IO uint8_t *) spixbase;
        // zacne se prijimat dalsi hodnota (aby se nezdrzovalo)
        *(__IO uint8_t *) spixbase = 0x00;

        // pokud jsou testovaci data
        #if defined(TEST_DATA)
          spiData = testDataNumber;
        #endif

        // dalsi hodnota vyzvednuta / vyslana
        v++;
        // pokud se vysila pouze 16bits a je to 3byte -> tak jej zahodit
        if ((adsBits16or24 == 0) && (v == 3)) { v = 0; continue; }

        // ulozeni jako dve ASCII-HEX cisla
        *buffer_write_ptr++ = bin2hex[(spiData >> 4) & 0x0F];
        *buffer_write_ptr++ = bin2hex[spiData & 0x0F];
    }

    // cekani na dokonceni prijmu na SPI
    while (!(SPI1->SR & SPI_FLAG_RXNE));
    // vyzvednuti hodnoty prijate z SPI
    spiData = *(__IO uint8_t *) spixbase;

    // pokud jsou testovaci data
    #if defined(TEST_DATA)
      spiData = testDataNumber;
    #endif

    // posledni hodnota se prenasi pouze pokud je nastavno '24bits'
    if (adsBits16or24 == 1)
    {
        // ulozeni jako dve ASCII-HEX cisla
        *buffer_write_ptr++ = bin2hex[(spiData >> 4) & 0x0F];
        *buffer_write_ptr++ = bin2hex[spiData & 0x0F];
    }
    */

    // vlozeni konce paketu ('paket status' a 'koncovy znak')
    *buffer_write_ptr++ = dataSendStatus;
    *buffer_write_ptr++ = PAKET_END;

    // dalsi paket pridan do vystupniho bufferu
    USBBufferInPtr += paketLenght;
    // test zda neby pridan posledni paket na konec bufferu
    if (USBBufferInPtr >= USBBufferAddressMax)
        { USBBufferInPtr = 0; }
    // v bufferu je dalsi paket
    USBBufferPacketsCount++;

    // at byl jakykoli 'DataSendStatus', tak prste bude 'DataSendStatusOK'
    dataSendStatus = PaketDataStatusOK;
    // nulovani poctu ztracenych pakeu / dat
    adsDataLostCount = 0;

    // pocitani poctu vyslanych paketu / sad dat
    #if defined(TEST_DATA_COUNT)
      adsDataSendCount++;
    #endif
    // inkrement cisla pro testovaci data (paket obsahuje stejna cisla)
    #if defined(TEST_DATA)
      testDataNumber++;
    #endif
    // mereni casu 'us' trvani sestaveni jednoho datoveho paketu
    #if defined(TEST_TIME_TO_SEND)
      testTimeCreatePaket = TIM_GetCounter(TIM16) - testTimeCreatePaket;
    #endif
}

// --- ---

// volano pri preplneni vystupniho bufferu na USB
void USBBufferIsFull()
{
    // ve vysilacim bufferu neni volno, je plny, nevisila se
    dataSendStatus = PaketDataStatusBufFull;
    // dalsi paket ztracen
    adsDataLostCount++;
    // pokud je ztraceni vice paketu nez 2s mereni, tak konec vysilani
    if (adsDataLostCount > (adsSpeed * 2))
    {
        // kontinualni prenos dat zastaven
        localFlags &= ~LocFlagADSDataContinuous;
        // 'Error' blika s touto rychlosti
        dataSendErrNextTimeOnOffLED = uwTick + BlickTimeOnUSBDataStop;
        // priznak varovneho blikani
        localFlags |= LocFlagDataSendErr;
        // zacne se rozsvicenim 'Error'
        ErrorOnLED();
    }
}

// vynulovani USB bufferu (pro vkladani i vyzvedavani)
void USBBufferFlush()
{
    adsDataReady = 0;
    USBBufferInPtr = USBBufferOutPtr = 0;
    USBBufferPacketsCount = 0;
}


