
#ifndef MAIN_APP_H_
#define MAIN_APP_H_

#include "GlobalDef.h"

// lokalni priznaky pro rizeni programu
typedef enum _FlagsLocal
{
    LocFlagAction = (uint32_t)(1 << 0),
    LocFlagRunningAsk = (uint32_t)(1 << 5),       // pozadavek na spusteni
    LocFlagRunning = (uint32_t)(1 << 6),          // spusteno
    LocFlagADSDataContinuous = (uint32_t)(1 << 9),
    LocFlagADSDataReady = (uint32_t)(1 << 10),
    LocFlagDataSendErr = (uint32_t)(1 << 11),     // (uplne) zastaveni vysiani dat z ADS do USB
} FlagsLocal;

typedef enum _ADSRegAddress
{
    ADSRegAdrReadStart = 0,
    ADSRegAdrID = 0,
    ADSRegAdrWriteStart = 1,
    ADSRegAdrCON1 = 1,
    ADSRegAdrCON2 = 2,
    ADSRegAdrCON3 = 3,
    ADSRegAdrLOFF = 4,
    ADSRegAdrCH1SET = 5,
    ADSRegAdrCH2SET = 6,
    ADSRegAdrCH3SET = 7,
    ADSRegAdrCH4SET = 8,
    ADSRegAdrCH5SET = 9,
    ADSRegAdrCH6SET = 10,
    ADSRegAdrCH7SET = 11,
    ADSRegAdrCH8SET = 12,
    ADSRegAdrRDL_SENSP = 13,
    ADSRegAdrRDL_SENSN = 14,
    ADSRegAdrLOFF_SENSP = 15,
    ADSRegAdrLOFF_SENSN = 16,
    ADSRegAdrLOFF_FLIP = 17,
    ADSRegAdrLOFF_STATP = 18,
    ADSRegAdrLOFF_STATN = 19,
    ADSRegAdrGPIO = 20,
    ADSRegAdrPACE = 21,
    ADSRegAdrRESP = 22,
    ADSRegAdrCONFIG4 = 23,
    ADSRegAdrWTC1 = 24,
    ADSRegAdrWTC2 = 25,
    ADSRegAdrReadEnd = 25,
    ADSRegAdrWriteEnd = 25,
    ADSRegAdrEXT1 = 26, // 16/24bit
    ADSRegAdrExtEnd = 26
} ADSRegAddress;

typedef struct _GlobalConfig
{
    uint8_t DevNum;     // identifikacni cislo tohoto zarizeni
} GlobalConfig;

// ulozena akce z komunikace (pro vykonani)
typedef enum
{
    ActFromCmnNone = 0,           // zadna akce
    ActFromCmnResetAppSW = 1,     // SW Reset aplikace
    ActFromCmnResetAppHW = 2,     // HW Reset aplikace
    ActFromCmnResetBootHW = 3,    // Reset pro vstup do 'BootLoader'
} ActionFromCommun;

typedef enum _SPI_CS_STATE
{
    SPI_CS_NONE = 0x00,
    SPI_CS_START = 0x01,
    SPI_CS_END = 0x02
} SPI_CS_STATE;

// data send 'Status'
typedef enum
{
    PaketDataStatusOK = (uint8_t)'0',
    PaketDataStatusBufFull = (uint8_t)'1',
    PaketDataStatusTimeOver = (uint8_t)'2',
} PaketDataStatus;

// --- externe definovana data ---

// Info
extern const char InfoDevProd[];
extern const char InfoDevName[];
extern const char InfoHWType[];
extern const char InfoHWVer[];
extern const char InfoHWRev[];
extern const char InfoSWType[];
extern const char InfoSWVer[];
extern const char InfoSWRev[];
extern const uint8_t InfoSWUid[];

extern uint8_t actionFromCommun;

extern GlobalConfig globalConfig;

extern uint8_t AdsConfig[27];      // konfigurace pro ADS

extern uint8_t adsChannels;        // pocet aktivnich kanalu (vzdy ve vyznamu 0, 1, 2, ...)
extern uint8_t adsBits16or24;      // zda se meri a prenasi 0 -> 16bit / 1 -> 24bit
extern int adsSpeed;         // pocet vzorku za vterinu

extern uint8_t adsDataReady;       // pocet detekovanych pulsu 'DataReady' z ADS
extern int16_t adsDataSendInfoOnOffLED;       // pocet paketu
extern int16_t adsDataLostCount;   // pocet kontinualne zreacenych paketu (nezapsanych do USB bufferu)

extern uint32_t localFlags;        // priznaky programu

extern uint8_t adsBuffer[MeasureBytes];
extern int paketLenght;
extern int USBBufferAddressMax;       // maximalni adresa v bufferu
extern int USBBufferPacketsMax;       // maximaln pocet paketu v bufferu
extern int USBBufferPacketsCount;     // aktualni pocet paketu v bufferu

extern void AdsConfigToAdsChip();
extern void AdsChipToAdsConfig();
extern void AdsConfigToMeasureParams();

// modra LED (akce)
void ActionOnLED();
void ActionOffLED();
void ActionToggleLED();
// cervena LED (error)
void ErrorOnLED();
void ErrorOffLED();
void ErrorToggleLED();

void SPI_CS_HIGH();
void SPI_CS_LOW();
uint8_t SPI_SEND_RECEIVE_8BIT(uint8_t dataIn, uint8_t action);

void ADS_RESET_PIN_SET(GPIO_PinState state);
void ADS_SLEEP_PIN_SET(GPIO_PinState state);
void ADS_START_PIN_SET(GPIO_PinState state);
uint8_t ADS_READY_PIN_GET();

void MyTestTimerInit(int value);
void MyTestTimerStart();
void MyTestTimerStop();

void MyInit();  // inicializace pred 'Loop'
void MyLoop();  // volano z hlavni 'Loop'

void SendDataToUSB();
void USBBufferFlush();

#endif /* MAIN_APP */
