#include <stdint.h>
#include <string.h>
#include "GlobalDef.h"

extern SetupConfig setupConfig[];

int dataValue = 0;  // pro vyzvedavani hodnot z paketu

// buffer pro zpracovani prijateho paketu
byte MyComPaketReceiveBuffer[MY_COM_BUFFER_IN_SIZE];
// bufer pro tvoreni odesilaneho paketu
byte MyComPaketSendBuffer[MY_COM_BUFFER_OUT_SIZE];

// inicialiace komunikace
void MyComInit()
{
    // nastaveni bufferu pro prijem z vysilani
    CommunInit(MyComPaketReceiveBuffer, MyComPaketSendBuffer);
}

// prijem dat (odkaz na buffer a odkaz na pocet)
void MyComReceived(uint8_t* Buf, uint32_t *Len)
{
    // zpracovani prijatych dat
    // (odkaz na 'buffer', pocet dat / hodnot v 'buffer')
    PaketRecvTestBufer(Buf, *Len);
    // pokud byl prijat cely paket -> 'CommunRecDone = 1'
}

// zpracovani prijateho paketu (pokud je)
byte MyComPaketProcess()
{
    // pokud neni prijat zadny cely paket, tak navrat
    if(!(CommunFlags & CommunRecDone)) { return 0; }

    // nejprve se do-vyslou vsechna data z USB bufferu
    while(USBBufferPacketsCount > 0) { SendDataToUSB(); }

    // zpracovani prijateho paketu
    return PaketProcess(MyComPaketSendBuffer);
}

// --- ---

// zpracovani jednotlivych CMDs
byte MyComPaketCmd()
{
    // defaultne se predpoklada odpoved
    CommunFlags |= CommunSendAns;

    // CMD - Reset (HW/SW)
    if (cmdValue == PaketCmdReset)
    {
        // vyzvednuti sub-cmd
        int subCmd = PaketRecvGetDataNumber('X');
        // pokud je to 'SW Reset'
        if (subCmd == ResetSubCmdSoft) { actionFromCommun = ActFromCmnResetAppSW; }
            // jinak je to 'HW Reset' aplikace (0,1)
            else { actionFromCommun = ActFromCmnResetAppHW; }
        // vykonano (???)
        cmdStatus = PaketStatusOK;
    }
    // CMD - Info
    else if (cmdValue == PaketCmdInfo)
    {
        // vyzvednuti sub-cmd
        int subCmd = PaketRecvGetDataNumber('X');
        // Device 'Info'
        if (subCmd == InfoSubCmdDev)
        {
            // producent, Nazev
            PaketSendAddDataNameAndString(&paketSendBufferActual, 'P', InfoDevProd);
            PaketSendAddDataNameAndString(&paketSendBufferActual, 'N', InfoDevName);
        }
        // HW 'Info'
        if (subCmd == InfoSubCmdHW)
        {
            // 'typ', 'verze', 'revize'
            PaketSendAddDataNameAndString(&paketSendBufferActual, 'T', InfoHWType);
            PaketSendAddDataNameAndString(&paketSendBufferActual, 'V', InfoHWVer);
            PaketSendAddDataNameAndString(&paketSendBufferActual, 'R', InfoHWRev);
            // vlozeni 'HW Uid' dat do paketu
            PaketSendAddDataNameAndNumberHexPtr(&paketSendBufferActual, 'U', STM32_UUID_UINT8_PTR, -12);
        }
        // SW 'Info'
        if (subCmd == InfoSubCmdSW)
        {
            // 'typ', 'verze', 'revize'
            PaketSendAddDataNameAndString(&paketSendBufferActual, 'T', InfoSWType);
            PaketSendAddDataNameAndString(&paketSendBufferActual, 'V', InfoSWVer);
            PaketSendAddDataNameAndString(&paketSendBufferActual, 'R', InfoSWRev);
            // vlozeni 'SW Uid' dat do paketu
            //PaketSendAddDataNameAndString(&paketSendBufferActual, 'U', InfoSWUid);
            PaketSendAddDataNameAndNumberHexPtr(&paketSendBufferActual, 'U', (uint8_t*)InfoSWUid, 10);
        }
        cmdStatus = PaketStatusOK;
    }
    // CMD - Init
    else if (cmdValue == PaketCmdInit)
    {
        cmdStatus = PaketStatusOK;
    }
    // CMD - Close
    else if (cmdValue == PaketCmdClose)
    {
        cmdStatus = PaketStatusOK;
    }
    // CMD - Start
    else if (cmdValue == PaketCmdStart)
    {
        // aktualni stanoveni parametru pro mereni
        AdsConfigToMeasureParams();

        // test generovani paketu / dat pomoci casovace
        #if defined(TEST_BY_TIMER)
          // rychlost casovace podle poctu vzorku
          MyTestTimerInit(1000000/adsSpeed);
        #endif

        // poadavek na spusteni behu
        localFlags |= LocFlagRunningAsk;
        // (beh bude spusten az pozdeji po odeslani odpovedi)
        cmdStatus = PaketStatusOK;
    }
    // CMD - Stop
    else if (cmdValue == PaketCmdStop)
    {
        // konci kontinualni mereni
        localFlags &= ~LocFlagADSDataContinuous;
        // zastaveni behu
        localFlags &= ~LocFlagRunning;

        // test generovani paketu / dat pomoci casovace
        #if defined(TEST_BY_TIMER)
          // zastaveni casovace
          MyTestTimerStop();
        #else
          // skutecna data
          // konci komunikace pres SPI
          SPI_CS_HIGH();
          // zastaveni mereni v ADS
          SPI_SEND_RECEIVE_8BIT(0x11, SPI_CS_START | SPI_CS_END);
          // ukonceni mereni v ADS (START = 0)
          ADS_START_PIN_SET(GPIO_PIN_RESET); // START
        #endif
        // radeji pockej (???)
        HAL_Delay(1000);

        // zhasnuti LEDs
        ActionOffLED(); ErrorOffLED();
        // vycisteni USB bufferu
        USBBufferFlush();
        // zadost o sestaveni odpovedi
        cmdStatus = PaketStatusOK;
    }
    // CMD - Check
    else if (cmdValue == PaketCmdCheck)
    {
        // cislo zarizeni
        PaketSendAddDataNameAndNumber(&paketSendBufferActual, 'N', globalConfig.DevNum);
        // zda se meri (1 - bezi / 0 - nebezi)
        PaketSendAddDataNameAndNumber(&paketSendBufferActual, 'R', (localFlags & LocFlagRunning) ? 1 : 0);

        cmdStatus = PaketStatusOK;
    }
    // CMD - DevSet (zakladni nastaveni zarizeni)
    else if (cmdValue == PaketCmdDevSet)
    {
        // vyzvednuti cisla zarizeni a jeho ulozeni
        globalConfig.DevNum = (uint8_t)PaketRecvGetDataNumber('N');
        // v 'GlobalConfig' byla zmena
        setupConfig[0].flags |= SETUP_FLAG_CHANGED;
        // ulozit 'GlobalConfig' do EEPROM/FLASH
        SetupStore((void*)&globalConfig);
    }
    // CMD - Setup (ulozeni nastaveni)
    else if (cmdValue == PaketCmdSetup)
    {
        // vyzvednuti sub-cmd
        int subCmd = PaketRecvGetDataNumber('X');
        // 'Setup' -> 'EEProm'
        if (subCmd == SetupSubCmdStore) { SetupStore(NULL); }
        // 'EEProm' -> 'Setup'
        else if (subCmd == SetupSubCmdLoad) { SetupLoad(NULL); }
        // 'EEProm' / 'Setup' -> Clear
        else if (subCmd == SetupSubCmdClear)
            { SetupClear(); SetupInit(); }
        cmdStatus = PaketStatusOK;
    }
    // CMD - Update (aktualizace)
    else if (cmdValue == PaketCmdUpdate)
    {
        // 'HW Reset' do 'BootLoader'
        actionFromCommun = ActFromCmnResetBootHW;
        // hotovo (odpoved jeste pred vykonanim)
        cmdStatus = PaketStatusOK;
    }

    // parametry 'ADSko'
    // zapis do registru
    else if (cmdValue == PaketCmdSetParams)
    {
        // od teto adresy
        uint8_t regsAddress = PaketRecvGetDataNumber('A');
        // tolik hodnot
        uint8_t regsCount = PaketRecvGetDataNumber('C');
        // zacatek dat v paketu pro zapis do konfigurace 'ADS'
        PaketRecvGetDataNamePos('D');

        // postupne vyzvednuti vsech dat pro konfiguraci
        for(int i = regsAddress; i <= regsCount; i++)
        {
            // vyzvednuti jedne hodnoty
            dataValue = PaketRecvGetDataNumberNext();
            // zapis do konfigurace
            AdsConfig[i] = dataValue;
        }
        // data se zmenila
        setupConfig[1].flags |= SETUP_FLAG_CHANGED;
        // skutecny zapis dat do 'ADS'
        AdsConfigToAdsChip();
        // hotovo
        cmdStatus = PaketStatusOK;
    }
    // cteni registru (ctou se vzdy vsechny registry)
    else if (cmdValue == PaketCmdGetParams)
    {
        // skutecne vycteni aktualniho stavu registru z 'ADS'
        AdsChipToAdsConfig();

        // od teto adresy
        PaketSendAddDataNameAndNumber(&paketSendBufferActual, 'A', ADSRegAdrID);
        // tolik hodnot
        PaketSendAddDataNameAndNumber(&paketSendBufferActual, 'C', 27);
        // sem se budou davat data do odesilaneho paketu
        PaketSendAddDataName(&paketSendBufferActual, 'D');

        // precteni vsech hodnot pro odeslani zpet
        for(int i = ADSRegAdrID; i <= ADSRegAdrExtEnd; i++)
        {
            // precteni jedne hodnoty
            dataValue = AdsConfig[i];
            // jeji zapis do odesilaneho paketu
            PaketSendAddDataNumber(&paketSendBufferActual, dataValue);
        }
        // hotovo
        cmdStatus = PaketStatusOK;
    }
    // zmereni a odeslani jedne sady dat
    else if (cmdValue == PaketCmdGetValue)
    {
        //SPI_CS_LOW();
        //GPIO_PinState pinState = ADS_READY_PIN_GET();

        // zacatek mereni 'Start'
        SPI_SEND_RECEIVE_8BIT(0x08, SPI_CS_START);
        // cekjen pokud ADS_READY == log0
        while(ADS_READY_PIN_GET() == GPIO_PIN_RESET);
        // cekjen pokud ADS_READY == log1
        while(ADS_READY_PIN_GET() == GPIO_PIN_SET);
        // zacatek vycitani dat
        SPI_SEND_RECEIVE_8BIT(0x12, SPI_CS_NONE);
        // postupne vycteni vsech dat
        for(int i = 0; i < MeasureBytes; i++)
        {
            // vycteni jednoho byte (z hodnoty)
            adsBuffer[i] = SPI_SEND_RECEIVE_8BIT(0x00, SPI_CS_NONE);
        }
        // konec mereni
        SPI_SEND_RECEIVE_8BIT(0x0A, SPI_CS_NONE);
        SPI_SEND_RECEIVE_8BIT(0x11, SPI_CS_END);

        // vlozeni dat do paketu ve tvaru AsciiHex
        PaketSendAddDataNameAndNumberHexPtr(&paketSendBufferActual, 'D', adsBuffer, MeasureBytes);
        // data jsou jiz v paketu
        cmdStatus = PaketStatusOK;
    }

    // --- povely specificke pro zarizeni ---

    // vycteni nekterych parametru hlavne pro zobrazeni
    else if (cmdValue == DeviceExtCmdMainParms)
    {
        // aktualni stanoveni pozadovanych parametru
        AdsConfigToMeasureParams();
        // vlozeni dat do odesilaneho paketu / odpovedi
        // aktivni kanaly (maji bity = 1)
        PaketSendAddDataNameAndNumber(&paketSendBufferActual, 'C', adsChannels);
        // presnost mereni
        PaketSendAddDataNameAndNumber(&paketSendBufferActual, 'P', adsBits16or24);
        // rychlost mereni
        PaketSendAddDataNameAndNumber(&paketSendBufferActual, 'S', adsSpeed);

        cmdStatus = PaketStatusOK;
    }
}

// zpracovani jednotlivych DATAs
uint8_t MyComPaketData() { }

// --- ---

// funkce pro skutecne odeslani paketu
extern uint8_t CDC_Transmit_FS(uint8_t* Buf, uint16_t Len);
// odeslani vytvoreneho paketu
void MyComPaketSend(byte *bufferData, int bufferLength)
{
    // skutecne odeslani paketu
    CDC_Transmit_FS(bufferData, bufferLength);
}
